# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

VOLUME_MERGE_METADATA = Metadata(
    id="fff7d7c8a2075faa5da5ccad34db6f9ba87459dd.workbench",
    name="volume-merge",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_VolumeMergeUpToParamsDictNoTag = typing.TypedDict('_VolumeMergeUpToParamsDictNoTag', {
    "last-subvol": str,
    "reverse": bool,
})
VolumeMergeUpToParamsDictTagged = typing.TypedDict('VolumeMergeUpToParamsDictTagged', {
    "@type": typing.Literal["up-to"],
    "last-subvol": str,
    "reverse": bool,
})
VolumeMergeUpToParamsDict = _VolumeMergeUpToParamsDictNoTag | VolumeMergeUpToParamsDictTagged


_VolumeMergeSubvolumeParamsDictNoTag = typing.TypedDict('_VolumeMergeSubvolumeParamsDictNoTag', {
    "subvol": str,
    "up-to": typing.NotRequired[VolumeMergeUpToParamsDict | None],
})
VolumeMergeSubvolumeParamsDictTagged = typing.TypedDict('VolumeMergeSubvolumeParamsDictTagged', {
    "@type": typing.Literal["subvolume"],
    "subvol": str,
    "up-to": typing.NotRequired[VolumeMergeUpToParamsDict | None],
})
VolumeMergeSubvolumeParamsDict = _VolumeMergeSubvolumeParamsDictNoTag | VolumeMergeSubvolumeParamsDictTagged


_VolumeMergeVolumeParamsDictNoTag = typing.TypedDict('_VolumeMergeVolumeParamsDictNoTag', {
    "volume-in": InputPathType,
    "subvolume": typing.NotRequired[list[VolumeMergeSubvolumeParamsDict] | None],
})
VolumeMergeVolumeParamsDictTagged = typing.TypedDict('VolumeMergeVolumeParamsDictTagged', {
    "@type": typing.Literal["volume"],
    "volume-in": InputPathType,
    "subvolume": typing.NotRequired[list[VolumeMergeSubvolumeParamsDict] | None],
})
VolumeMergeVolumeParamsDict = _VolumeMergeVolumeParamsDictNoTag | VolumeMergeVolumeParamsDictTagged


_VolumeMergeParamsDictNoTag = typing.TypedDict('_VolumeMergeParamsDictNoTag', {
    "volume-out": str,
    "volume": typing.NotRequired[list[VolumeMergeVolumeParamsDict] | None],
})
VolumeMergeParamsDictTagged = typing.TypedDict('VolumeMergeParamsDictTagged', {
    "@type": typing.Literal["workbench/volume-merge"],
    "volume-out": str,
    "volume": typing.NotRequired[list[VolumeMergeVolumeParamsDict] | None],
})
VolumeMergeParamsDict = _VolumeMergeParamsDictNoTag | VolumeMergeParamsDictTagged


def volume_merge_up_to(
    last_subvol: str,
    reverse: bool = False,
) -> VolumeMergeUpToParamsDictTagged:
    """
    Build parameters.
    
    Args:
        last_subvol: the number or name of the last subvolume to include.
        reverse: use the range in reverse order.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "up-to",
        "last-subvol": last_subvol,
        "reverse": reverse,
    }
    return params


def volume_merge_up_to_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `VolumeMergeUpToParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("last-subvol", None) is None:
        raise StyxValidationError("`last-subvol` must not be None")
    if not isinstance(params["last-subvol"], str):
        raise StyxValidationError(f'`last-subvol` has the wrong type: Received `{type(params.get("last-subvol", None))}` expected `str`')
    if params.get("reverse", False) is None:
        raise StyxValidationError("`reverse` must not be None")
    if not isinstance(params["reverse"], bool):
        raise StyxValidationError(f'`reverse` has the wrong type: Received `{type(params.get("reverse", False))}` expected `bool`')


def volume_merge_up_to_cargs(
    params: VolumeMergeUpToParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-up-to",
        params.get("last-subvol", None)
    ])
    if params.get("reverse", False):
        cargs.append("-reverse")
    return cargs


def volume_merge_subvolume(
    subvol: str,
    up_to: VolumeMergeUpToParamsDict | None = None,
) -> VolumeMergeSubvolumeParamsDictTagged:
    """
    Build parameters.
    
    Args:
        subvol: the subvolume number or name.
        up_to: use an inclusive range of subvolumes.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "subvolume",
        "subvol": subvol,
    }
    if up_to is not None:
        params["up-to"] = up_to
    return params


def volume_merge_subvolume_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `VolumeMergeSubvolumeParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("subvol", None) is None:
        raise StyxValidationError("`subvol` must not be None")
    if not isinstance(params["subvol"], str):
        raise StyxValidationError(f'`subvol` has the wrong type: Received `{type(params.get("subvol", None))}` expected `str`')
    if params.get("up-to", None) is not None:
        volume_merge_up_to_validate(params["up-to"])


def volume_merge_subvolume_cargs(
    params: VolumeMergeSubvolumeParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-subvolume",
        params.get("subvol", None),
        *(volume_merge_up_to_cargs(params.get("up-to", None), execution) if (params.get("up-to", None) is not None) else [])
    ])
    return cargs


def volume_merge_volume(
    volume_in: InputPathType,
    subvolume: list[VolumeMergeSubvolumeParamsDict] | None = None,
) -> VolumeMergeVolumeParamsDictTagged:
    """
    Build parameters.
    
    Args:
        volume_in: a volume file to use subvolumes from.
        subvolume: select a single subvolume to use.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "volume",
        "volume-in": volume_in,
    }
    if subvolume is not None:
        params["subvolume"] = subvolume
    return params


def volume_merge_volume_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `VolumeMergeVolumeParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("volume-in", None) is None:
        raise StyxValidationError("`volume-in` must not be None")
    if not isinstance(params["volume-in"], (pathlib.Path, str)):
        raise StyxValidationError(f'`volume-in` has the wrong type: Received `{type(params.get("volume-in", None))}` expected `InputPathType`')
    if params.get("subvolume", None) is not None:
        if not isinstance(params["subvolume"], list):
            raise StyxValidationError(f'`subvolume` has the wrong type: Received `{type(params.get("subvolume", None))}` expected `list[VolumeMergeSubvolumeParamsDict] | None`')
        for e in params["subvolume"]:
            volume_merge_subvolume_validate(e)


def volume_merge_volume_cargs(
    params: VolumeMergeVolumeParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-volume",
        execution.input_file(params.get("volume-in", None)),
        *([a for c in [volume_merge_subvolume_cargs(s, execution) for s in params.get("subvolume", None)] for a in c] if (params.get("subvolume", None) is not None) else [])
    ])
    return cargs


class VolumeMergeOutputs(typing.NamedTuple):
    """
    Output object returned when calling `VolumeMergeParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    volume_out: OutputPathType
    """the output volume file"""


def volume_merge_params(
    volume_out: str,
    volume: list[VolumeMergeVolumeParamsDict] | None = None,
) -> VolumeMergeParamsDictTagged:
    """
    Build parameters.
    
    Args:
        volume_out: the output volume file.
        volume: specify an input volume file.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/volume-merge",
        "volume-out": volume_out,
    }
    if volume is not None:
        params["volume"] = volume
    return params


def volume_merge_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `VolumeMergeParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("volume-out", None) is None:
        raise StyxValidationError("`volume-out` must not be None")
    if not isinstance(params["volume-out"], str):
        raise StyxValidationError(f'`volume-out` has the wrong type: Received `{type(params.get("volume-out", None))}` expected `str`')
    if params.get("volume", None) is not None:
        if not isinstance(params["volume"], list):
            raise StyxValidationError(f'`volume` has the wrong type: Received `{type(params.get("volume", None))}` expected `list[VolumeMergeVolumeParamsDict] | None`')
        for e in params["volume"]:
            volume_merge_volume_validate(e)


def volume_merge_cargs(
    params: VolumeMergeParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-volume-merge"
    ])
    cargs.append(params.get("volume-out", None))
    if params.get("volume", None) is not None:
        cargs.extend([a for c in [volume_merge_volume_cargs(s, execution) for s in params.get("volume", None)] for a in c])
    return cargs


def volume_merge_outputs(
    params: VolumeMergeParamsDict,
    execution: Execution,
) -> VolumeMergeOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = VolumeMergeOutputs(
        root=execution.output_file("."),
        volume_out=execution.output_file(params.get("volume-out", None)),
    )
    return ret


def volume_merge_execute(
    params: VolumeMergeParamsDict,
    runner: Runner | None = None,
) -> VolumeMergeOutputs:
    """
    MERGE VOLUME FILES INTO A NEW FILE.
    
    Takes one or more volume files and constructs a new volume file by
    concatenating subvolumes from them. The input volume files must have the
    same volume space.
    
    Example: wb_command -volume-merge out.nii -volume first.nii -subvolume 1
    -volume second.nii
    
    This example would take the first subvolume from first.nii, followed by all
    subvolumes from second.nii, and write these to out.nii.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VolumeMergeOutputs`).
    """
    volume_merge_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(VOLUME_MERGE_METADATA)
    params = execution.params(params)
    cargs = volume_merge_cargs(params, execution)
    ret = volume_merge_outputs(params, execution)
    execution.run(cargs)
    return ret


def volume_merge(
    volume_out: str,
    volume: list[VolumeMergeVolumeParamsDict] | None = None,
    runner: Runner | None = None,
) -> VolumeMergeOutputs:
    """
    MERGE VOLUME FILES INTO A NEW FILE.
    
    Takes one or more volume files and constructs a new volume file by
    concatenating subvolumes from them. The input volume files must have the
    same volume space.
    
    Example: wb_command -volume-merge out.nii -volume first.nii -subvolume 1
    -volume second.nii
    
    This example would take the first subvolume from first.nii, followed by all
    subvolumes from second.nii, and write these to out.nii.
    
    Args:
        volume_out: the output volume file.
        volume: specify an input volume file.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VolumeMergeOutputs`).
    """
    params = volume_merge_params(
        volume_out=volume_out,
        volume=volume,
    )
    return volume_merge_execute(params, runner)


__all__ = [
    "VOLUME_MERGE_METADATA",
    "VolumeMergeOutputs",
    "VolumeMergeParamsDict",
    "VolumeMergeParamsDictTagged",
    "VolumeMergeSubvolumeParamsDict",
    "VolumeMergeSubvolumeParamsDictTagged",
    "VolumeMergeUpToParamsDict",
    "VolumeMergeUpToParamsDictTagged",
    "VolumeMergeVolumeParamsDict",
    "VolumeMergeVolumeParamsDictTagged",
    "volume_merge",
    "volume_merge_execute",
    "volume_merge_params",
    "volume_merge_subvolume",
    "volume_merge_up_to",
    "volume_merge_volume",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

VOLUME_DILATE_METADATA = Metadata(
    id="63f0ca3ad59f9e1f8239c3ea5c2e72dfdc878c7e.workbench",
    name="volume-dilate",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_VolumeDilatePresmoothParamsDictNoTag = typing.TypedDict('_VolumeDilatePresmoothParamsDictNoTag', {
    "kernel": float,
    "fwhm": bool,
})
VolumeDilatePresmoothParamsDictTagged = typing.TypedDict('VolumeDilatePresmoothParamsDictTagged', {
    "@type": typing.Literal["presmooth"],
    "kernel": float,
    "fwhm": bool,
})
VolumeDilatePresmoothParamsDict = _VolumeDilatePresmoothParamsDictNoTag | VolumeDilatePresmoothParamsDictTagged


_VolumeDilateGradExtrapolateParamsDictNoTag = typing.TypedDict('_VolumeDilateGradExtrapolateParamsDictNoTag', {
    "presmooth": typing.NotRequired[VolumeDilatePresmoothParamsDict | None],
})
VolumeDilateGradExtrapolateParamsDictTagged = typing.TypedDict('VolumeDilateGradExtrapolateParamsDictTagged', {
    "@type": typing.Literal["grad-extrapolate"],
    "presmooth": typing.NotRequired[VolumeDilatePresmoothParamsDict | None],
})
VolumeDilateGradExtrapolateParamsDict = _VolumeDilateGradExtrapolateParamsDictNoTag | VolumeDilateGradExtrapolateParamsDictTagged


_VolumeDilateParamsDictNoTag = typing.TypedDict('_VolumeDilateParamsDictNoTag', {
    "volume": InputPathType,
    "distance": float,
    "method": str,
    "volume-out": str,
    "grad-extrapolate": typing.NotRequired[VolumeDilateGradExtrapolateParamsDict | None],
    "subvol": typing.NotRequired[str | None],
    "roi-volume": typing.NotRequired[InputPathType | None],
    "roi-volume": typing.NotRequired[InputPathType | None],
    "exponent": typing.NotRequired[float | None],
    "legacy-cutoff": bool,
})
VolumeDilateParamsDictTagged = typing.TypedDict('VolumeDilateParamsDictTagged', {
    "@type": typing.Literal["workbench/volume-dilate"],
    "volume": InputPathType,
    "distance": float,
    "method": str,
    "volume-out": str,
    "grad-extrapolate": typing.NotRequired[VolumeDilateGradExtrapolateParamsDict | None],
    "subvol": typing.NotRequired[str | None],
    "roi-volume": typing.NotRequired[InputPathType | None],
    "roi-volume": typing.NotRequired[InputPathType | None],
    "exponent": typing.NotRequired[float | None],
    "legacy-cutoff": bool,
})
VolumeDilateParamsDict = _VolumeDilateParamsDictNoTag | VolumeDilateParamsDictTagged


def volume_dilate_presmooth(
    kernel: float,
    fwhm: bool = False,
) -> VolumeDilatePresmoothParamsDictTagged:
    """
    Build parameters.
    
    Args:
        kernel: the size of gaussian smoothing kernel in mm, as sigma by\
            default.
        fwhm: kernel size is FWHM, not sigma.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "presmooth",
        "kernel": kernel,
        "fwhm": fwhm,
    }
    return params


def volume_dilate_presmooth_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `VolumeDilatePresmoothParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("kernel", None) is None:
        raise StyxValidationError("`kernel` must not be None")
    if not isinstance(params["kernel"], (float, int)):
        raise StyxValidationError(f'`kernel` has the wrong type: Received `{type(params.get("kernel", None))}` expected `float`')
    if params.get("fwhm", False) is None:
        raise StyxValidationError("`fwhm` must not be None")
    if not isinstance(params["fwhm"], bool):
        raise StyxValidationError(f'`fwhm` has the wrong type: Received `{type(params.get("fwhm", False))}` expected `bool`')


def volume_dilate_presmooth_cargs(
    params: VolumeDilatePresmoothParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-presmooth",
        str(params.get("kernel", None))
    ])
    if params.get("fwhm", False):
        cargs.append("-fwhm")
    return cargs


def volume_dilate_grad_extrapolate(
    presmooth: VolumeDilatePresmoothParamsDict | None = None,
) -> VolumeDilateGradExtrapolateParamsDictTagged:
    """
    Build parameters.
    
    Args:
        presmooth: apply presmoothing before computing gradient vectors, not\
            recommended.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "grad-extrapolate",
    }
    if presmooth is not None:
        params["presmooth"] = presmooth
    return params


def volume_dilate_grad_extrapolate_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `VolumeDilateGradExtrapolateParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("presmooth", None) is not None:
        volume_dilate_presmooth_validate(params["presmooth"])


def volume_dilate_grad_extrapolate_cargs(
    params: VolumeDilateGradExtrapolateParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    if params.get("presmooth", None) is not None:
        cargs.extend([
            "-grad-extrapolate",
            *volume_dilate_presmooth_cargs(params.get("presmooth", None), execution)
        ])
    return cargs


class VolumeDilateOutputs(typing.NamedTuple):
    """
    Output object returned when calling `VolumeDilateParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    volume_out: OutputPathType
    """the output volume"""


def volume_dilate_params(
    volume: InputPathType,
    distance: float,
    method: str,
    volume_out: str,
    grad_extrapolate: VolumeDilateGradExtrapolateParamsDict | None = None,
    subvol: str | None = None,
    roi_volume: InputPathType | None = None,
    roi_volume_: InputPathType | None = None,
    exponent: float | None = None,
    legacy_cutoff: bool = False,
) -> VolumeDilateParamsDictTagged:
    """
    Build parameters.
    
    Args:
        volume: the volume to dilate.
        distance: distance in mm to dilate.
        method: dilation method to use.
        volume_out: the output volume.
        grad_extrapolate: additionally use the gradient to extrapolate,\
            intended to be used with WEIGHTED.
        subvol: select a single subvolume to dilate\
            \
            the subvolume number or name.
        roi_volume: specify an roi of where there is data\
            \
            volume file, positive values denote voxels that have data.
        roi_volume_: specify an roi of voxels to overwrite, rather than voxels\
            with value zero\
            \
            volume file, positive values denote voxels to have their values\
            replaced.
        exponent: use a different exponent in the weighting function\
            \
            exponent 'n' to use in (1 / (distance ^ n)) as the weighting\
            function (default 7).
        legacy_cutoff: use the v1.3.2 method of excluding voxels further than\
            the dilation distance when calculating the dilated value.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/volume-dilate",
        "volume": volume,
        "distance": distance,
        "method": method,
        "volume-out": volume_out,
        "legacy-cutoff": legacy_cutoff,
    }
    if grad_extrapolate is not None:
        params["grad-extrapolate"] = grad_extrapolate
    if subvol is not None:
        params["subvol"] = subvol
    if roi_volume is not None:
        params["roi-volume"] = roi_volume
    if roi_volume_ is not None:
        params["roi-volume"] = roi_volume_
    if exponent is not None:
        params["exponent"] = exponent
    return params


def volume_dilate_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `VolumeDilateParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("volume", None) is None:
        raise StyxValidationError("`volume` must not be None")
    if not isinstance(params["volume"], (pathlib.Path, str)):
        raise StyxValidationError(f'`volume` has the wrong type: Received `{type(params.get("volume", None))}` expected `InputPathType`')
    if params.get("distance", None) is None:
        raise StyxValidationError("`distance` must not be None")
    if not isinstance(params["distance"], (float, int)):
        raise StyxValidationError(f'`distance` has the wrong type: Received `{type(params.get("distance", None))}` expected `float`')
    if params.get("method", None) is None:
        raise StyxValidationError("`method` must not be None")
    if not isinstance(params["method"], str):
        raise StyxValidationError(f'`method` has the wrong type: Received `{type(params.get("method", None))}` expected `str`')
    if params.get("volume-out", None) is None:
        raise StyxValidationError("`volume-out` must not be None")
    if not isinstance(params["volume-out"], str):
        raise StyxValidationError(f'`volume-out` has the wrong type: Received `{type(params.get("volume-out", None))}` expected `str`')
    if params.get("grad-extrapolate", None) is not None:
        volume_dilate_grad_extrapolate_validate(params["grad-extrapolate"])
    if params.get("subvol", None) is not None:
        if not isinstance(params["subvol"], str):
            raise StyxValidationError(f'`subvol` has the wrong type: Received `{type(params.get("subvol", None))}` expected `str | None`')
    if params.get("roi-volume", None) is not None:
        if not isinstance(params["roi-volume"], (pathlib.Path, str)):
            raise StyxValidationError(f'`roi-volume` has the wrong type: Received `{type(params.get("roi-volume", None))}` expected `InputPathType | None`')
    if params.get("roi-volume", None) is not None:
        if not isinstance(params["roi-volume"], (pathlib.Path, str)):
            raise StyxValidationError(f'`roi-volume` has the wrong type: Received `{type(params.get("roi-volume", None))}` expected `InputPathType | None`')
    if params.get("exponent", None) is not None:
        if not isinstance(params["exponent"], (float, int)):
            raise StyxValidationError(f'`exponent` has the wrong type: Received `{type(params.get("exponent", None))}` expected `float | None`')
    if params.get("legacy-cutoff", False) is None:
        raise StyxValidationError("`legacy-cutoff` must not be None")
    if not isinstance(params["legacy-cutoff"], bool):
        raise StyxValidationError(f'`legacy-cutoff` has the wrong type: Received `{type(params.get("legacy-cutoff", False))}` expected `bool`')


def volume_dilate_cargs(
    params: VolumeDilateParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-volume-dilate"
    ])
    cargs.append(execution.input_file(params.get("volume", None)))
    cargs.append(str(params.get("distance", None)))
    cargs.append(params.get("method", None))
    cargs.append(params.get("volume-out", None))
    if params.get("grad-extrapolate", None) is not None:
        cargs.extend(volume_dilate_grad_extrapolate_cargs(params.get("grad-extrapolate", None), execution))
    if params.get("subvol", None) is not None:
        cargs.extend([
            "-subvolume",
            params.get("subvol", None)
        ])
    if params.get("roi-volume", None) is not None:
        cargs.extend([
            "-data-roi",
            execution.input_file(params.get("roi-volume", None))
        ])
    if params.get("roi-volume", None) is not None:
        cargs.extend([
            "-bad-voxel-roi",
            execution.input_file(params.get("roi-volume", None))
        ])
    if params.get("exponent", None) is not None:
        cargs.extend([
            "-exponent",
            str(params.get("exponent", None))
        ])
    if params.get("legacy-cutoff", False):
        cargs.append("-legacy-cutoff")
    return cargs


def volume_dilate_outputs(
    params: VolumeDilateParamsDict,
    execution: Execution,
) -> VolumeDilateOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = VolumeDilateOutputs(
        root=execution.output_file("."),
        volume_out=execution.output_file(params.get("volume-out", None)),
    )
    return ret


def volume_dilate_execute(
    params: VolumeDilateParamsDict,
    runner: Runner | None = None,
) -> VolumeDilateOutputs:
    """
    DILATE A VOLUME FILE.
    
    For all voxels that are designated as bad, if they neighbor a non-bad voxel
    with data or are within the specified distance of such a voxel, replace the
    value in the bad voxel with a value calculated from nearby non-bad voxels
    that have data, otherwise set the value to zero. No matter how small
    <distance> is, dilation will always use at least the face neighbor voxels.
    
    By default, voxels that have data with the value 0 are bad, specify
    -bad-voxel-roi to only count voxels as bad if they are selected by the roi.
    If -data-roi is not specified, all voxels are assumed to have data.
    
    To get the behavior of version 1.3.2 or earlier, use '-legacy-cutoff
    -exponent 2'.
    
    Valid values for <method> are:
    
    NEAREST - use the value from the nearest good voxel
    WEIGHTED - use a weighted average based on distance.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VolumeDilateOutputs`).
    """
    volume_dilate_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(VOLUME_DILATE_METADATA)
    params = execution.params(params)
    cargs = volume_dilate_cargs(params, execution)
    ret = volume_dilate_outputs(params, execution)
    execution.run(cargs)
    return ret


def volume_dilate(
    volume: InputPathType,
    distance: float,
    method: str,
    volume_out: str,
    grad_extrapolate: VolumeDilateGradExtrapolateParamsDict | None = None,
    subvol: str | None = None,
    roi_volume: InputPathType | None = None,
    roi_volume_: InputPathType | None = None,
    exponent: float | None = None,
    legacy_cutoff: bool = False,
    runner: Runner | None = None,
) -> VolumeDilateOutputs:
    """
    DILATE A VOLUME FILE.
    
    For all voxels that are designated as bad, if they neighbor a non-bad voxel
    with data or are within the specified distance of such a voxel, replace the
    value in the bad voxel with a value calculated from nearby non-bad voxels
    that have data, otherwise set the value to zero. No matter how small
    <distance> is, dilation will always use at least the face neighbor voxels.
    
    By default, voxels that have data with the value 0 are bad, specify
    -bad-voxel-roi to only count voxels as bad if they are selected by the roi.
    If -data-roi is not specified, all voxels are assumed to have data.
    
    To get the behavior of version 1.3.2 or earlier, use '-legacy-cutoff
    -exponent 2'.
    
    Valid values for <method> are:
    
    NEAREST - use the value from the nearest good voxel
    WEIGHTED - use a weighted average based on distance.
    
    Args:
        volume: the volume to dilate.
        distance: distance in mm to dilate.
        method: dilation method to use.
        volume_out: the output volume.
        grad_extrapolate: additionally use the gradient to extrapolate,\
            intended to be used with WEIGHTED.
        subvol: select a single subvolume to dilate\
            \
            the subvolume number or name.
        roi_volume: specify an roi of where there is data\
            \
            volume file, positive values denote voxels that have data.
        roi_volume_: specify an roi of voxels to overwrite, rather than voxels\
            with value zero\
            \
            volume file, positive values denote voxels to have their values\
            replaced.
        exponent: use a different exponent in the weighting function\
            \
            exponent 'n' to use in (1 / (distance ^ n)) as the weighting\
            function (default 7).
        legacy_cutoff: use the v1.3.2 method of excluding voxels further than\
            the dilation distance when calculating the dilated value.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VolumeDilateOutputs`).
    """
    params = volume_dilate_params(
        volume=volume,
        distance=distance,
        method=method,
        volume_out=volume_out,
        grad_extrapolate=grad_extrapolate,
        subvol=subvol,
        roi_volume=roi_volume,
        roi_volume_=roi_volume_,
        exponent=exponent,
        legacy_cutoff=legacy_cutoff,
    )
    return volume_dilate_execute(params, runner)


__all__ = [
    "VOLUME_DILATE_METADATA",
    "VolumeDilateGradExtrapolateParamsDict",
    "VolumeDilateGradExtrapolateParamsDictTagged",
    "VolumeDilateOutputs",
    "VolumeDilateParamsDict",
    "VolumeDilateParamsDictTagged",
    "VolumeDilatePresmoothParamsDict",
    "VolumeDilatePresmoothParamsDictTagged",
    "volume_dilate",
    "volume_dilate_execute",
    "volume_dilate_grad_extrapolate",
    "volume_dilate_params",
    "volume_dilate_presmooth",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

BACKEND_AVERAGE_DENSE_ROI_METADATA = Metadata(
    id="af0de4e5ef947f744f8051acd2cc8811f3f4cf73.workbench",
    name="backend-average-dense-roi",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_BackendAverageDenseRoiParamsDictNoTag = typing.TypedDict('_BackendAverageDenseRoiParamsDictNoTag', {
    "index-list": str,
    "out-file": str,
})
BackendAverageDenseRoiParamsDictTagged = typing.TypedDict('BackendAverageDenseRoiParamsDictTagged', {
    "@type": typing.Literal["workbench/backend-average-dense-roi"],
    "index-list": str,
    "out-file": str,
})
BackendAverageDenseRoiParamsDict = _BackendAverageDenseRoiParamsDictNoTag | BackendAverageDenseRoiParamsDictTagged


class BackendAverageDenseRoiOutputs(typing.NamedTuple):
    """
    Output object returned when calling `BackendAverageDenseRoiParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def backend_average_dense_roi_params(
    index_list: str,
    out_file: str,
) -> BackendAverageDenseRoiParamsDictTagged:
    """
    Build parameters.
    
    Args:
        index_list: comma separated list of cifti indexes to average.
        out_file: file to write the average row to.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/backend-average-dense-roi",
        "index-list": index_list,
        "out-file": out_file,
    }
    return params


def backend_average_dense_roi_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `BackendAverageDenseRoiParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("index-list", None) is None:
        raise StyxValidationError("`index-list` must not be None")
    if not isinstance(params["index-list"], str):
        raise StyxValidationError(f'`index-list` has the wrong type: Received `{type(params.get("index-list", None))}` expected `str`')
    if params.get("out-file", None) is None:
        raise StyxValidationError("`out-file` must not be None")
    if not isinstance(params["out-file"], str):
        raise StyxValidationError(f'`out-file` has the wrong type: Received `{type(params.get("out-file", None))}` expected `str`')


def backend_average_dense_roi_cargs(
    params: BackendAverageDenseRoiParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-backend-average-dense-roi"
    ])
    cargs.append(params.get("index-list", None))
    cargs.append(params.get("out-file", None))
    return cargs


def backend_average_dense_roi_outputs(
    params: BackendAverageDenseRoiParamsDict,
    execution: Execution,
) -> BackendAverageDenseRoiOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = BackendAverageDenseRoiOutputs(
        root=execution.output_file("."),
    )
    return ret


def backend_average_dense_roi_execute(
    params: BackendAverageDenseRoiParamsDict,
    runner: Runner | None = None,
) -> BackendAverageDenseRoiOutputs:
    """
    CONNECTOME DB BACKEND COMMAND FOR CIFTI AVERAGE DENSE ROI.
    
    This command is probably not the one you are looking for, try
    -cifti-average-dense-roi. It takes the list of cifti files to average from
    standard input, and writes its output as little endian, 32-bit integer of
    row size followed by the row as 32-bit floats.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `BackendAverageDenseRoiOutputs`).
    """
    backend_average_dense_roi_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(BACKEND_AVERAGE_DENSE_ROI_METADATA)
    params = execution.params(params)
    cargs = backend_average_dense_roi_cargs(params, execution)
    ret = backend_average_dense_roi_outputs(params, execution)
    execution.run(cargs)
    return ret


def backend_average_dense_roi(
    index_list: str,
    out_file: str,
    runner: Runner | None = None,
) -> BackendAverageDenseRoiOutputs:
    """
    CONNECTOME DB BACKEND COMMAND FOR CIFTI AVERAGE DENSE ROI.
    
    This command is probably not the one you are looking for, try
    -cifti-average-dense-roi. It takes the list of cifti files to average from
    standard input, and writes its output as little endian, 32-bit integer of
    row size followed by the row as 32-bit floats.
    
    Args:
        index_list: comma separated list of cifti indexes to average.
        out_file: file to write the average row to.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `BackendAverageDenseRoiOutputs`).
    """
    params = backend_average_dense_roi_params(
        index_list=index_list,
        out_file=out_file,
    )
    return backend_average_dense_roi_execute(params, runner)


__all__ = [
    "BACKEND_AVERAGE_DENSE_ROI_METADATA",
    "BackendAverageDenseRoiOutputs",
    "BackendAverageDenseRoiParamsDict",
    "BackendAverageDenseRoiParamsDictTagged",
    "backend_average_dense_roi",
    "backend_average_dense_roi_execute",
    "backend_average_dense_roi_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

GIFTI_LABEL_TO_ROI_METADATA = Metadata(
    id="82549fa4f4d713245781e307c4dd75baad9fe47c.workbench",
    name="gifti-label-to-roi",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_GiftiLabelToRoiParamsDictNoTag = typing.TypedDict('_GiftiLabelToRoiParamsDictNoTag', {
    "label-in": InputPathType,
    "metric-out": str,
    "map": typing.NotRequired[str | None],
    "label-key": typing.NotRequired[int | None],
    "label-name": typing.NotRequired[str | None],
})
GiftiLabelToRoiParamsDictTagged = typing.TypedDict('GiftiLabelToRoiParamsDictTagged', {
    "@type": typing.Literal["workbench/gifti-label-to-roi"],
    "label-in": InputPathType,
    "metric-out": str,
    "map": typing.NotRequired[str | None],
    "label-key": typing.NotRequired[int | None],
    "label-name": typing.NotRequired[str | None],
})
GiftiLabelToRoiParamsDict = _GiftiLabelToRoiParamsDictNoTag | GiftiLabelToRoiParamsDictTagged


class GiftiLabelToRoiOutputs(typing.NamedTuple):
    """
    Output object returned when calling `GiftiLabelToRoiParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    metric_out: OutputPathType
    """the output metric file"""


def gifti_label_to_roi_params(
    label_in: InputPathType,
    metric_out: str,
    map_: str | None = None,
    label_key: int | None = None,
    label_name: str | None = None,
) -> GiftiLabelToRoiParamsDictTagged:
    """
    Build parameters.
    
    Args:
        label_in: the input gifti label file.
        metric_out: the output metric file.
        map_: select a single label map to use\
            \
            the map number or name.
        label_key: select label by key\
            \
            the label key that you want an roi of.
        label_name: select label by name\
            \
            the label name that you want an roi of.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/gifti-label-to-roi",
        "label-in": label_in,
        "metric-out": metric_out,
    }
    if map_ is not None:
        params["map"] = map_
    if label_key is not None:
        params["label-key"] = label_key
    if label_name is not None:
        params["label-name"] = label_name
    return params


def gifti_label_to_roi_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `GiftiLabelToRoiParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("label-in", None) is None:
        raise StyxValidationError("`label-in` must not be None")
    if not isinstance(params["label-in"], (pathlib.Path, str)):
        raise StyxValidationError(f'`label-in` has the wrong type: Received `{type(params.get("label-in", None))}` expected `InputPathType`')
    if params.get("metric-out", None) is None:
        raise StyxValidationError("`metric-out` must not be None")
    if not isinstance(params["metric-out"], str):
        raise StyxValidationError(f'`metric-out` has the wrong type: Received `{type(params.get("metric-out", None))}` expected `str`')
    if params.get("map", None) is not None:
        if not isinstance(params["map"], str):
            raise StyxValidationError(f'`map` has the wrong type: Received `{type(params.get("map", None))}` expected `str | None`')
    if params.get("label-key", None) is not None:
        if not isinstance(params["label-key"], int):
            raise StyxValidationError(f'`label-key` has the wrong type: Received `{type(params.get("label-key", None))}` expected `int | None`')
    if params.get("label-name", None) is not None:
        if not isinstance(params["label-name"], str):
            raise StyxValidationError(f'`label-name` has the wrong type: Received `{type(params.get("label-name", None))}` expected `str | None`')


def gifti_label_to_roi_cargs(
    params: GiftiLabelToRoiParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-gifti-label-to-roi"
    ])
    cargs.append(execution.input_file(params.get("label-in", None)))
    cargs.append(params.get("metric-out", None))
    if params.get("map", None) is not None:
        cargs.extend([
            "-map",
            params.get("map", None)
        ])
    if params.get("label-key", None) is not None:
        cargs.extend([
            "-key",
            str(params.get("label-key", None))
        ])
    if params.get("label-name", None) is not None:
        cargs.extend([
            "-name",
            params.get("label-name", None)
        ])
    return cargs


def gifti_label_to_roi_outputs(
    params: GiftiLabelToRoiParamsDict,
    execution: Execution,
) -> GiftiLabelToRoiOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = GiftiLabelToRoiOutputs(
        root=execution.output_file("."),
        metric_out=execution.output_file(params.get("metric-out", None)),
    )
    return ret


def gifti_label_to_roi_execute(
    params: GiftiLabelToRoiParamsDict,
    runner: Runner | None = None,
) -> GiftiLabelToRoiOutputs:
    """
    MAKE A GIFTI LABEL INTO AN ROI METRIC.
    
    For each map in <label-in>, a map is created in <metric-out> where all
    locations labeled with <label-name> or with a key of <label-key> are given a
    value of 1, and all other locations are given 0. Exactly one of -name and
    -key must be specified. Specify -map to use only one map from <label-in>.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `GiftiLabelToRoiOutputs`).
    """
    gifti_label_to_roi_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(GIFTI_LABEL_TO_ROI_METADATA)
    params = execution.params(params)
    cargs = gifti_label_to_roi_cargs(params, execution)
    ret = gifti_label_to_roi_outputs(params, execution)
    execution.run(cargs)
    return ret


def gifti_label_to_roi(
    label_in: InputPathType,
    metric_out: str,
    map_: str | None = None,
    label_key: int | None = None,
    label_name: str | None = None,
    runner: Runner | None = None,
) -> GiftiLabelToRoiOutputs:
    """
    MAKE A GIFTI LABEL INTO AN ROI METRIC.
    
    For each map in <label-in>, a map is created in <metric-out> where all
    locations labeled with <label-name> or with a key of <label-key> are given a
    value of 1, and all other locations are given 0. Exactly one of -name and
    -key must be specified. Specify -map to use only one map from <label-in>.
    
    Args:
        label_in: the input gifti label file.
        metric_out: the output metric file.
        map_: select a single label map to use\
            \
            the map number or name.
        label_key: select label by key\
            \
            the label key that you want an roi of.
        label_name: select label by name\
            \
            the label name that you want an roi of.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `GiftiLabelToRoiOutputs`).
    """
    params = gifti_label_to_roi_params(
        label_in=label_in,
        metric_out=metric_out,
        map_=map_,
        label_key=label_key,
        label_name=label_name,
    )
    return gifti_label_to_roi_execute(params, runner)


__all__ = [
    "GIFTI_LABEL_TO_ROI_METADATA",
    "GiftiLabelToRoiOutputs",
    "GiftiLabelToRoiParamsDict",
    "GiftiLabelToRoiParamsDictTagged",
    "gifti_label_to_roi",
    "gifti_label_to_roi_execute",
    "gifti_label_to_roi_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

VOLUME_LABEL_PROBABILITY_METADATA = Metadata(
    id="433915b90ed4936671e31ff5e9d775b5a9ffb4e9.workbench",
    name="volume-label-probability",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_VolumeLabelProbabilityParamsDictNoTag = typing.TypedDict('_VolumeLabelProbabilityParamsDictNoTag', {
    "label-maps": InputPathType,
    "probability-out": str,
    "exclude-unlabeled": bool,
})
VolumeLabelProbabilityParamsDictTagged = typing.TypedDict('VolumeLabelProbabilityParamsDictTagged', {
    "@type": typing.Literal["workbench/volume-label-probability"],
    "label-maps": InputPathType,
    "probability-out": str,
    "exclude-unlabeled": bool,
})
VolumeLabelProbabilityParamsDict = _VolumeLabelProbabilityParamsDictNoTag | VolumeLabelProbabilityParamsDictTagged


class VolumeLabelProbabilityOutputs(typing.NamedTuple):
    """
    Output object returned when calling `VolumeLabelProbabilityParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    probability_out: OutputPathType
    """the relative frequencies of each label at each voxel"""


def volume_label_probability_params(
    label_maps: InputPathType,
    probability_out: str,
    exclude_unlabeled: bool = False,
) -> VolumeLabelProbabilityParamsDictTagged:
    """
    Build parameters.
    
    Args:
        label_maps: volume label file containing individual label maps from\
            many subjects.
        probability_out: the relative frequencies of each label at each voxel.
        exclude_unlabeled: don't make a probability map of the unlabeled key.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/volume-label-probability",
        "label-maps": label_maps,
        "probability-out": probability_out,
        "exclude-unlabeled": exclude_unlabeled,
    }
    return params


def volume_label_probability_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `VolumeLabelProbabilityParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("label-maps", None) is None:
        raise StyxValidationError("`label-maps` must not be None")
    if not isinstance(params["label-maps"], (pathlib.Path, str)):
        raise StyxValidationError(f'`label-maps` has the wrong type: Received `{type(params.get("label-maps", None))}` expected `InputPathType`')
    if params.get("probability-out", None) is None:
        raise StyxValidationError("`probability-out` must not be None")
    if not isinstance(params["probability-out"], str):
        raise StyxValidationError(f'`probability-out` has the wrong type: Received `{type(params.get("probability-out", None))}` expected `str`')
    if params.get("exclude-unlabeled", False) is None:
        raise StyxValidationError("`exclude-unlabeled` must not be None")
    if not isinstance(params["exclude-unlabeled"], bool):
        raise StyxValidationError(f'`exclude-unlabeled` has the wrong type: Received `{type(params.get("exclude-unlabeled", False))}` expected `bool`')


def volume_label_probability_cargs(
    params: VolumeLabelProbabilityParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-volume-label-probability"
    ])
    cargs.append(execution.input_file(params.get("label-maps", None)))
    cargs.append(params.get("probability-out", None))
    if params.get("exclude-unlabeled", False):
        cargs.append("-exclude-unlabeled")
    return cargs


def volume_label_probability_outputs(
    params: VolumeLabelProbabilityParamsDict,
    execution: Execution,
) -> VolumeLabelProbabilityOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = VolumeLabelProbabilityOutputs(
        root=execution.output_file("."),
        probability_out=execution.output_file(params.get("probability-out", None)),
    )
    return ret


def volume_label_probability_execute(
    params: VolumeLabelProbabilityParamsDict,
    runner: Runner | None = None,
) -> VolumeLabelProbabilityOutputs:
    """
    FIND FREQUENCY OF VOLUME LABELS.
    
    This command outputs a set of soft ROIs, one for each label in the input,
    where the value is how many of the input maps had that label at that voxel,
    divided by the number of input maps.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VolumeLabelProbabilityOutputs`).
    """
    volume_label_probability_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(VOLUME_LABEL_PROBABILITY_METADATA)
    params = execution.params(params)
    cargs = volume_label_probability_cargs(params, execution)
    ret = volume_label_probability_outputs(params, execution)
    execution.run(cargs)
    return ret


def volume_label_probability(
    label_maps: InputPathType,
    probability_out: str,
    exclude_unlabeled: bool = False,
    runner: Runner | None = None,
) -> VolumeLabelProbabilityOutputs:
    """
    FIND FREQUENCY OF VOLUME LABELS.
    
    This command outputs a set of soft ROIs, one for each label in the input,
    where the value is how many of the input maps had that label at that voxel,
    divided by the number of input maps.
    
    Args:
        label_maps: volume label file containing individual label maps from\
            many subjects.
        probability_out: the relative frequencies of each label at each voxel.
        exclude_unlabeled: don't make a probability map of the unlabeled key.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VolumeLabelProbabilityOutputs`).
    """
    params = volume_label_probability_params(
        label_maps=label_maps,
        probability_out=probability_out,
        exclude_unlabeled=exclude_unlabeled,
    )
    return volume_label_probability_execute(params, runner)


__all__ = [
    "VOLUME_LABEL_PROBABILITY_METADATA",
    "VolumeLabelProbabilityOutputs",
    "VolumeLabelProbabilityParamsDict",
    "VolumeLabelProbabilityParamsDictTagged",
    "volume_label_probability",
    "volume_label_probability_execute",
    "volume_label_probability_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

SPEC_FILE_MERGE_METADATA = Metadata(
    id="85ecceaff1fc0a1530a0d4627dab53d2a72a20ea.workbench",
    name="spec-file-merge",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_SpecFileMergeParamsDictNoTag = typing.TypedDict('_SpecFileMergeParamsDictNoTag', {
    "spec-1": str,
    "spec-2": str,
    "out-spec": str,
})
SpecFileMergeParamsDictTagged = typing.TypedDict('SpecFileMergeParamsDictTagged', {
    "@type": typing.Literal["workbench/spec-file-merge"],
    "spec-1": str,
    "spec-2": str,
    "out-spec": str,
})
SpecFileMergeParamsDict = _SpecFileMergeParamsDictNoTag | SpecFileMergeParamsDictTagged


class SpecFileMergeOutputs(typing.NamedTuple):
    """
    Output object returned when calling `SpecFileMergeParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def spec_file_merge_params(
    spec_1: str,
    spec_2: str,
    out_spec: str,
) -> SpecFileMergeParamsDictTagged:
    """
    Build parameters.
    
    Args:
        spec_1: first spec file to merge.
        spec_2: second spec file to merge.
        out_spec: output - output spec file.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/spec-file-merge",
        "spec-1": spec_1,
        "spec-2": spec_2,
        "out-spec": out_spec,
    }
    return params


def spec_file_merge_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `SpecFileMergeParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("spec-1", None) is None:
        raise StyxValidationError("`spec-1` must not be None")
    if not isinstance(params["spec-1"], str):
        raise StyxValidationError(f'`spec-1` has the wrong type: Received `{type(params.get("spec-1", None))}` expected `str`')
    if params.get("spec-2", None) is None:
        raise StyxValidationError("`spec-2` must not be None")
    if not isinstance(params["spec-2"], str):
        raise StyxValidationError(f'`spec-2` has the wrong type: Received `{type(params.get("spec-2", None))}` expected `str`')
    if params.get("out-spec", None) is None:
        raise StyxValidationError("`out-spec` must not be None")
    if not isinstance(params["out-spec"], str):
        raise StyxValidationError(f'`out-spec` has the wrong type: Received `{type(params.get("out-spec", None))}` expected `str`')


def spec_file_merge_cargs(
    params: SpecFileMergeParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-spec-file-merge"
    ])
    cargs.append(params.get("spec-1", None))
    cargs.append(params.get("spec-2", None))
    cargs.append(params.get("out-spec", None))
    return cargs


def spec_file_merge_outputs(
    params: SpecFileMergeParamsDict,
    execution: Execution,
) -> SpecFileMergeOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = SpecFileMergeOutputs(
        root=execution.output_file("."),
    )
    return ret


def spec_file_merge_execute(
    params: SpecFileMergeParamsDict,
    runner: Runner | None = None,
) -> SpecFileMergeOutputs:
    """
    MERGE TWO SPEC FILES INTO ONE.
    
    The output spec file contains every file that is in either of the input spec
    files.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SpecFileMergeOutputs`).
    """
    spec_file_merge_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(SPEC_FILE_MERGE_METADATA)
    params = execution.params(params)
    cargs = spec_file_merge_cargs(params, execution)
    ret = spec_file_merge_outputs(params, execution)
    execution.run(cargs)
    return ret


def spec_file_merge(
    spec_1: str,
    spec_2: str,
    out_spec: str,
    runner: Runner | None = None,
) -> SpecFileMergeOutputs:
    """
    MERGE TWO SPEC FILES INTO ONE.
    
    The output spec file contains every file that is in either of the input spec
    files.
    
    Args:
        spec_1: first spec file to merge.
        spec_2: second spec file to merge.
        out_spec: output - output spec file.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SpecFileMergeOutputs`).
    """
    params = spec_file_merge_params(
        spec_1=spec_1,
        spec_2=spec_2,
        out_spec=out_spec,
    )
    return spec_file_merge_execute(params, runner)


__all__ = [
    "SPEC_FILE_MERGE_METADATA",
    "SpecFileMergeOutputs",
    "SpecFileMergeParamsDict",
    "SpecFileMergeParamsDictTagged",
    "spec_file_merge",
    "spec_file_merge_execute",
    "spec_file_merge_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

SURFACE_AFFINE_REGRESSION_METADATA = Metadata(
    id="81e681b772379bf62c291d6e75c808098051eb2f.workbench",
    name="surface-affine-regression",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_SurfaceAffineRegressionParamsDictNoTag = typing.TypedDict('_SurfaceAffineRegressionParamsDictNoTag', {
    "source": InputPathType,
    "target": InputPathType,
    "affine-out": str,
})
SurfaceAffineRegressionParamsDictTagged = typing.TypedDict('SurfaceAffineRegressionParamsDictTagged', {
    "@type": typing.Literal["workbench/surface-affine-regression"],
    "source": InputPathType,
    "target": InputPathType,
    "affine-out": str,
})
SurfaceAffineRegressionParamsDict = _SurfaceAffineRegressionParamsDictNoTag | SurfaceAffineRegressionParamsDictTagged


class SurfaceAffineRegressionOutputs(typing.NamedTuple):
    """
    Output object returned when calling `SurfaceAffineRegressionParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def surface_affine_regression_params(
    source: InputPathType,
    target: InputPathType,
    affine_out: str,
) -> SurfaceAffineRegressionParamsDictTagged:
    """
    Build parameters.
    
    Args:
        source: the surface to warp.
        target: the surface to match the coordinates of.
        affine_out: output - the output affine file.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/surface-affine-regression",
        "source": source,
        "target": target,
        "affine-out": affine_out,
    }
    return params


def surface_affine_regression_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `SurfaceAffineRegressionParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("source", None) is None:
        raise StyxValidationError("`source` must not be None")
    if not isinstance(params["source"], (pathlib.Path, str)):
        raise StyxValidationError(f'`source` has the wrong type: Received `{type(params.get("source", None))}` expected `InputPathType`')
    if params.get("target", None) is None:
        raise StyxValidationError("`target` must not be None")
    if not isinstance(params["target"], (pathlib.Path, str)):
        raise StyxValidationError(f'`target` has the wrong type: Received `{type(params.get("target", None))}` expected `InputPathType`')
    if params.get("affine-out", None) is None:
        raise StyxValidationError("`affine-out` must not be None")
    if not isinstance(params["affine-out"], str):
        raise StyxValidationError(f'`affine-out` has the wrong type: Received `{type(params.get("affine-out", None))}` expected `str`')


def surface_affine_regression_cargs(
    params: SurfaceAffineRegressionParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-surface-affine-regression"
    ])
    cargs.append(execution.input_file(params.get("source", None)))
    cargs.append(execution.input_file(params.get("target", None)))
    cargs.append(params.get("affine-out", None))
    return cargs


def surface_affine_regression_outputs(
    params: SurfaceAffineRegressionParamsDict,
    execution: Execution,
) -> SurfaceAffineRegressionOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = SurfaceAffineRegressionOutputs(
        root=execution.output_file("."),
    )
    return ret


def surface_affine_regression_execute(
    params: SurfaceAffineRegressionParamsDict,
    runner: Runner | None = None,
) -> SurfaceAffineRegressionOutputs:
    """
    REGRESS THE AFFINE TRANSFORM BETWEEN SURFACES ON THE SAME MESH.
    
    Use linear regression to compute an affine that minimizes the sum of squares
    of the coordinate differences between the target surface and the warped
    source surface. Note that this has a bias to shrink the surface that is
    being warped. The output is written as a NIFTI 'world' matrix, see
    -convert-affine to convert it for use in other software.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SurfaceAffineRegressionOutputs`).
    """
    surface_affine_regression_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(SURFACE_AFFINE_REGRESSION_METADATA)
    params = execution.params(params)
    cargs = surface_affine_regression_cargs(params, execution)
    ret = surface_affine_regression_outputs(params, execution)
    execution.run(cargs)
    return ret


def surface_affine_regression(
    source: InputPathType,
    target: InputPathType,
    affine_out: str,
    runner: Runner | None = None,
) -> SurfaceAffineRegressionOutputs:
    """
    REGRESS THE AFFINE TRANSFORM BETWEEN SURFACES ON THE SAME MESH.
    
    Use linear regression to compute an affine that minimizes the sum of squares
    of the coordinate differences between the target surface and the warped
    source surface. Note that this has a bias to shrink the surface that is
    being warped. The output is written as a NIFTI 'world' matrix, see
    -convert-affine to convert it for use in other software.
    
    Args:
        source: the surface to warp.
        target: the surface to match the coordinates of.
        affine_out: output - the output affine file.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SurfaceAffineRegressionOutputs`).
    """
    params = surface_affine_regression_params(
        source=source,
        target=target,
        affine_out=affine_out,
    )
    return surface_affine_regression_execute(params, runner)


__all__ = [
    "SURFACE_AFFINE_REGRESSION_METADATA",
    "SurfaceAffineRegressionOutputs",
    "SurfaceAffineRegressionParamsDict",
    "SurfaceAffineRegressionParamsDictTagged",
    "surface_affine_regression",
    "surface_affine_regression_execute",
    "surface_affine_regression_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

CIFTI_CREATE_PARCELLATED_FROM_TEMPLATE_METADATA = Metadata(
    id="3f274d723b32862f2a3c97e709a1481f947e0288.workbench",
    name="cifti-create-parcellated-from-template",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_CiftiCreateParcellatedFromTemplateCiftiParamsDictNoTag = typing.TypedDict('_CiftiCreateParcellatedFromTemplateCiftiParamsDictNoTag', {
    "cifti-in": InputPathType,
})
CiftiCreateParcellatedFromTemplateCiftiParamsDictTagged = typing.TypedDict('CiftiCreateParcellatedFromTemplateCiftiParamsDictTagged', {
    "@type": typing.Literal["cifti"],
    "cifti-in": InputPathType,
})
CiftiCreateParcellatedFromTemplateCiftiParamsDict = _CiftiCreateParcellatedFromTemplateCiftiParamsDictNoTag | CiftiCreateParcellatedFromTemplateCiftiParamsDictTagged


_CiftiCreateParcellatedFromTemplateParamsDictNoTag = typing.TypedDict('_CiftiCreateParcellatedFromTemplateParamsDictNoTag', {
    "cifti-template": InputPathType,
    "modify-direction": str,
    "cifti-out": str,
    "cifti": typing.NotRequired[list[CiftiCreateParcellatedFromTemplateCiftiParamsDict] | None],
    "value": typing.NotRequired[float | None],
})
CiftiCreateParcellatedFromTemplateParamsDictTagged = typing.TypedDict('CiftiCreateParcellatedFromTemplateParamsDictTagged', {
    "@type": typing.Literal["workbench/cifti-create-parcellated-from-template"],
    "cifti-template": InputPathType,
    "modify-direction": str,
    "cifti-out": str,
    "cifti": typing.NotRequired[list[CiftiCreateParcellatedFromTemplateCiftiParamsDict] | None],
    "value": typing.NotRequired[float | None],
})
CiftiCreateParcellatedFromTemplateParamsDict = _CiftiCreateParcellatedFromTemplateParamsDictNoTag | CiftiCreateParcellatedFromTemplateParamsDictTagged


def cifti_create_parcellated_from_template_cifti(
    cifti_in: InputPathType,
) -> CiftiCreateParcellatedFromTemplateCiftiParamsDictTagged:
    """
    Build parameters.
    
    Args:
        cifti_in: the input parcellated cifti file.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "cifti",
        "cifti-in": cifti_in,
    }
    return params


def cifti_create_parcellated_from_template_cifti_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiCreateParcellatedFromTemplateCiftiParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("cifti-in", None) is None:
        raise StyxValidationError("`cifti-in` must not be None")
    if not isinstance(params["cifti-in"], (pathlib.Path, str)):
        raise StyxValidationError(f'`cifti-in` has the wrong type: Received `{type(params.get("cifti-in", None))}` expected `InputPathType`')


def cifti_create_parcellated_from_template_cifti_cargs(
    params: CiftiCreateParcellatedFromTemplateCiftiParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-cifti",
        execution.input_file(params.get("cifti-in", None))
    ])
    return cargs


class CiftiCreateParcellatedFromTemplateOutputs(typing.NamedTuple):
    """
    Output object returned when calling `CiftiCreateParcellatedFromTemplateParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    cifti_out: OutputPathType
    """the output cifti file"""


def cifti_create_parcellated_from_template_params(
    cifti_template: InputPathType,
    modify_direction: str,
    cifti_out: str,
    cifti: list[CiftiCreateParcellatedFromTemplateCiftiParamsDict] | None = None,
    value: float | None = None,
) -> CiftiCreateParcellatedFromTemplateParamsDictTagged:
    """
    Build parameters.
    
    Args:
        cifti_template: a cifti file with the template parcel mapping along\
            column.
        modify_direction: which dimension of the output file should match the\
            template (integer, 'ROW', or 'COLUMN').
        cifti_out: the output cifti file.
        cifti: specify an input cifti file.
        value: specify value to be used in parcels that don't match\
            \
            value to use (default 0).
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/cifti-create-parcellated-from-template",
        "cifti-template": cifti_template,
        "modify-direction": modify_direction,
        "cifti-out": cifti_out,
    }
    if cifti is not None:
        params["cifti"] = cifti
    if value is not None:
        params["value"] = value
    return params


def cifti_create_parcellated_from_template_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiCreateParcellatedFromTemplateParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("cifti-template", None) is None:
        raise StyxValidationError("`cifti-template` must not be None")
    if not isinstance(params["cifti-template"], (pathlib.Path, str)):
        raise StyxValidationError(f'`cifti-template` has the wrong type: Received `{type(params.get("cifti-template", None))}` expected `InputPathType`')
    if params.get("modify-direction", None) is None:
        raise StyxValidationError("`modify-direction` must not be None")
    if not isinstance(params["modify-direction"], str):
        raise StyxValidationError(f'`modify-direction` has the wrong type: Received `{type(params.get("modify-direction", None))}` expected `str`')
    if params.get("cifti-out", None) is None:
        raise StyxValidationError("`cifti-out` must not be None")
    if not isinstance(params["cifti-out"], str):
        raise StyxValidationError(f'`cifti-out` has the wrong type: Received `{type(params.get("cifti-out", None))}` expected `str`')
    if params.get("cifti", None) is not None:
        if not isinstance(params["cifti"], list):
            raise StyxValidationError(f'`cifti` has the wrong type: Received `{type(params.get("cifti", None))}` expected `list[CiftiCreateParcellatedFromTemplateCiftiParamsDict] | None`')
        for e in params["cifti"]:
            cifti_create_parcellated_from_template_cifti_validate(e)
    if params.get("value", None) is not None:
        if not isinstance(params["value"], (float, int)):
            raise StyxValidationError(f'`value` has the wrong type: Received `{type(params.get("value", None))}` expected `float | None`')


def cifti_create_parcellated_from_template_cargs(
    params: CiftiCreateParcellatedFromTemplateParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-cifti-create-parcellated-from-template"
    ])
    cargs.append(execution.input_file(params.get("cifti-template", None)))
    cargs.append(params.get("modify-direction", None))
    cargs.append(params.get("cifti-out", None))
    if params.get("cifti", None) is not None:
        cargs.extend([a for c in [cifti_create_parcellated_from_template_cifti_cargs(s, execution) for s in params.get("cifti", None)] for a in c])
    if params.get("value", None) is not None:
        cargs.extend([
            "-fill-value",
            str(params.get("value", None))
        ])
    return cargs


def cifti_create_parcellated_from_template_outputs(
    params: CiftiCreateParcellatedFromTemplateParamsDict,
    execution: Execution,
) -> CiftiCreateParcellatedFromTemplateOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = CiftiCreateParcellatedFromTemplateOutputs(
        root=execution.output_file("."),
        cifti_out=execution.output_file(params.get("cifti-out", None)),
    )
    return ret


def cifti_create_parcellated_from_template_execute(
    params: CiftiCreateParcellatedFromTemplateParamsDict,
    runner: Runner | None = None,
) -> CiftiCreateParcellatedFromTemplateOutputs:
    """
    MATCH PARCELS TO TEMPLATE BY NAME.
    
    For each parcel name in the template mapping, find that name in an input
    cifti file and use its data in the output file. All input cifti files must
    have a parcels mapping along <modify-direction> and matching mappings along
    other dimensions. The direction can be either an integer starting from 1, or
    the strings 'ROW' or 'COLUMN'.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiCreateParcellatedFromTemplateOutputs`).
    """
    cifti_create_parcellated_from_template_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(CIFTI_CREATE_PARCELLATED_FROM_TEMPLATE_METADATA)
    params = execution.params(params)
    cargs = cifti_create_parcellated_from_template_cargs(params, execution)
    ret = cifti_create_parcellated_from_template_outputs(params, execution)
    execution.run(cargs)
    return ret


def cifti_create_parcellated_from_template(
    cifti_template: InputPathType,
    modify_direction: str,
    cifti_out: str,
    cifti: list[CiftiCreateParcellatedFromTemplateCiftiParamsDict] | None = None,
    value: float | None = None,
    runner: Runner | None = None,
) -> CiftiCreateParcellatedFromTemplateOutputs:
    """
    MATCH PARCELS TO TEMPLATE BY NAME.
    
    For each parcel name in the template mapping, find that name in an input
    cifti file and use its data in the output file. All input cifti files must
    have a parcels mapping along <modify-direction> and matching mappings along
    other dimensions. The direction can be either an integer starting from 1, or
    the strings 'ROW' or 'COLUMN'.
    
    Args:
        cifti_template: a cifti file with the template parcel mapping along\
            column.
        modify_direction: which dimension of the output file should match the\
            template (integer, 'ROW', or 'COLUMN').
        cifti_out: the output cifti file.
        cifti: specify an input cifti file.
        value: specify value to be used in parcels that don't match\
            \
            value to use (default 0).
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiCreateParcellatedFromTemplateOutputs`).
    """
    params = cifti_create_parcellated_from_template_params(
        cifti_template=cifti_template,
        modify_direction=modify_direction,
        cifti_out=cifti_out,
        cifti=cifti,
        value=value,
    )
    return cifti_create_parcellated_from_template_execute(params, runner)


__all__ = [
    "CIFTI_CREATE_PARCELLATED_FROM_TEMPLATE_METADATA",
    "CiftiCreateParcellatedFromTemplateCiftiParamsDict",
    "CiftiCreateParcellatedFromTemplateCiftiParamsDictTagged",
    "CiftiCreateParcellatedFromTemplateOutputs",
    "CiftiCreateParcellatedFromTemplateParamsDict",
    "CiftiCreateParcellatedFromTemplateParamsDictTagged",
    "cifti_create_parcellated_from_template",
    "cifti_create_parcellated_from_template_cifti",
    "cifti_create_parcellated_from_template_execute",
    "cifti_create_parcellated_from_template_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

SPEC_FILE_MODIFY_METADATA = Metadata(
    id="445d6412e50c348caabba3213e8b6d10e9cc1ae4.workbench",
    name="spec-file-modify",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_SpecFileModifyAddParamsDictNoTag = typing.TypedDict('_SpecFileModifyAddParamsDictNoTag', {
    "structure": str,
    "file": str,
})
SpecFileModifyAddParamsDictTagged = typing.TypedDict('SpecFileModifyAddParamsDictTagged', {
    "@type": typing.Literal["add"],
    "structure": str,
    "file": str,
})
SpecFileModifyAddParamsDict = _SpecFileModifyAddParamsDictNoTag | SpecFileModifyAddParamsDictTagged


_SpecFileModifyRemoveParamsDictNoTag = typing.TypedDict('_SpecFileModifyRemoveParamsDictNoTag', {
    "filename": str,
    "suffix": bool,
    "recursive": bool,
})
SpecFileModifyRemoveParamsDictTagged = typing.TypedDict('SpecFileModifyRemoveParamsDictTagged', {
    "@type": typing.Literal["remove"],
    "filename": str,
    "suffix": bool,
    "recursive": bool,
})
SpecFileModifyRemoveParamsDict = _SpecFileModifyRemoveParamsDictNoTag | SpecFileModifyRemoveParamsDictTagged


_SpecFileModifyParamsDictNoTag = typing.TypedDict('_SpecFileModifyParamsDictNoTag', {
    "spec-file": str,
    "add": typing.NotRequired[list[SpecFileModifyAddParamsDict] | None],
    "remove": typing.NotRequired[list[SpecFileModifyRemoveParamsDict] | None],
})
SpecFileModifyParamsDictTagged = typing.TypedDict('SpecFileModifyParamsDictTagged', {
    "@type": typing.Literal["workbench/spec-file-modify"],
    "spec-file": str,
    "add": typing.NotRequired[list[SpecFileModifyAddParamsDict] | None],
    "remove": typing.NotRequired[list[SpecFileModifyRemoveParamsDict] | None],
})
SpecFileModifyParamsDict = _SpecFileModifyParamsDictNoTag | SpecFileModifyParamsDictTagged


def spec_file_modify_add(
    structure: str,
    file: str,
) -> SpecFileModifyAddParamsDictTagged:
    """
    Build parameters.
    
    Args:
        structure: the structure of the data file.
        file: the file to add.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "add",
        "structure": structure,
        "file": file,
    }
    return params


def spec_file_modify_add_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `SpecFileModifyAddParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("structure", None) is None:
        raise StyxValidationError("`structure` must not be None")
    if not isinstance(params["structure"], str):
        raise StyxValidationError(f'`structure` has the wrong type: Received `{type(params.get("structure", None))}` expected `str`')
    if params.get("file", None) is None:
        raise StyxValidationError("`file` must not be None")
    if not isinstance(params["file"], str):
        raise StyxValidationError(f'`file` has the wrong type: Received `{type(params.get("file", None))}` expected `str`')


def spec_file_modify_add_cargs(
    params: SpecFileModifyAddParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-add",
        params.get("structure", None),
        params.get("file", None)
    ])
    return cargs


def spec_file_modify_remove(
    filename: str,
    suffix: bool = False,
    recursive: bool = False,
) -> SpecFileModifyRemoveParamsDictTagged:
    """
    Build parameters.
    
    Args:
        filename: the filename to remove.
        suffix: match any file that ends with the given string (if there are\
            multiple matches, error unless -recursive was also specified).
        recursive: remove all files that match the string starting from any\
            folder.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "remove",
        "filename": filename,
        "suffix": suffix,
        "recursive": recursive,
    }
    return params


def spec_file_modify_remove_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `SpecFileModifyRemoveParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("filename", None) is None:
        raise StyxValidationError("`filename` must not be None")
    if not isinstance(params["filename"], str):
        raise StyxValidationError(f'`filename` has the wrong type: Received `{type(params.get("filename", None))}` expected `str`')
    if params.get("suffix", False) is None:
        raise StyxValidationError("`suffix` must not be None")
    if not isinstance(params["suffix"], bool):
        raise StyxValidationError(f'`suffix` has the wrong type: Received `{type(params.get("suffix", False))}` expected `bool`')
    if params.get("recursive", False) is None:
        raise StyxValidationError("`recursive` must not be None")
    if not isinstance(params["recursive"], bool):
        raise StyxValidationError(f'`recursive` has the wrong type: Received `{type(params.get("recursive", False))}` expected `bool`')


def spec_file_modify_remove_cargs(
    params: SpecFileModifyRemoveParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-remove",
        params.get("filename", None)
    ])
    if params.get("suffix", False):
        cargs.append("-suffix")
    if params.get("recursive", False):
        cargs.append("-recursive")
    return cargs


class SpecFileModifyOutputs(typing.NamedTuple):
    """
    Output object returned when calling `SpecFileModifyParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def spec_file_modify_params(
    spec_file: str,
    add: list[SpecFileModifyAddParamsDict] | None = None,
    remove: list[SpecFileModifyRemoveParamsDict] | None = None,
) -> SpecFileModifyParamsDictTagged:
    """
    Build parameters.
    
    Args:
        spec_file: the spec file to modify.
        add: add a file to the spec.
        remove: remove a file from the spec file.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/spec-file-modify",
        "spec-file": spec_file,
    }
    if add is not None:
        params["add"] = add
    if remove is not None:
        params["remove"] = remove
    return params


def spec_file_modify_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `SpecFileModifyParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("spec-file", None) is None:
        raise StyxValidationError("`spec-file` must not be None")
    if not isinstance(params["spec-file"], str):
        raise StyxValidationError(f'`spec-file` has the wrong type: Received `{type(params.get("spec-file", None))}` expected `str`')
    if params.get("add", None) is not None:
        if not isinstance(params["add"], list):
            raise StyxValidationError(f'`add` has the wrong type: Received `{type(params.get("add", None))}` expected `list[SpecFileModifyAddParamsDict] | None`')
        for e in params["add"]:
            spec_file_modify_add_validate(e)
    if params.get("remove", None) is not None:
        if not isinstance(params["remove"], list):
            raise StyxValidationError(f'`remove` has the wrong type: Received `{type(params.get("remove", None))}` expected `list[SpecFileModifyRemoveParamsDict] | None`')
        for e in params["remove"]:
            spec_file_modify_remove_validate(e)


def spec_file_modify_cargs(
    params: SpecFileModifyParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-spec-file-modify"
    ])
    cargs.append(params.get("spec-file", None))
    if params.get("add", None) is not None or params.get("remove", None) is not None:
        cargs.extend([
            *([a for c in [spec_file_modify_add_cargs(s, execution) for s in params.get("add", None)] for a in c] if (params.get("add", None) is not None) else []),
            *([a for c in [spec_file_modify_remove_cargs(s, execution) for s in params.get("remove", None)] for a in c] if (params.get("remove", None) is not None) else [])
        ])
    return cargs


def spec_file_modify_outputs(
    params: SpecFileModifyParamsDict,
    execution: Execution,
) -> SpecFileModifyOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = SpecFileModifyOutputs(
        root=execution.output_file("."),
    )
    return ret


def spec_file_modify_execute(
    params: SpecFileModifyParamsDict,
    runner: Runner | None = None,
) -> SpecFileModifyOutputs:
    """
    ADD OR REMOVE FILES FROM SPEC FILE.
    
    The input spec file is overwritten with the new version. If the spec file
    does not exist, it is created with default metadata.
    
    Removals are done before additions, so that you can remove files with a
    pattern, and then add new files even if they match that removal pattern.
    
    The <structure> argument of -add must match one of the following strings
    (you can use INVALID for cifti or volume files):CORTEX_LEFT
    CORTEX_RIGHT
    CEREBELLUM
    ACCUMBENS_LEFT
    ACCUMBENS_RIGHT
    ALL_GREY_MATTER
    ALL_WHITE_MATTER
    AMYGDALA_LEFT
    AMYGDALA_RIGHT
    BRAIN_STEM
    CAUDATE_LEFT
    CAUDATE_RIGHT
    CEREBELLAR_WHITE_MATTER_LEFT
    CEREBELLAR_WHITE_MATTER_RIGHT
    CEREBELLUM_LEFT
    CEREBELLUM_RIGHT
    CEREBRAL_WHITE_MATTER_LEFT
    CEREBRAL_WHITE_MATTER_RIGHT
    CORTEX
    DIENCEPHALON_VENTRAL_LEFT
    DIENCEPHALON_VENTRAL_RIGHT
    HIPPOCAMPUS_LEFT
    HIPPOCAMPUS_RIGHT
    HIPPOCAMPUS_DENTATE_LEFT
    HIPPOCAMPUS_DENTATE_RIGHT
    INVALID
    OTHER
    OTHER_GREY_MATTER
    OTHER_WHITE_MATTER
    PALLIDUM_LEFT
    PALLIDUM_RIGHT
    PUTAMEN_LEFT
    PUTAMEN_RIGHT
    THALAMUS_LEFT
    THALAMUS_RIGHT
    .
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SpecFileModifyOutputs`).
    """
    spec_file_modify_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(SPEC_FILE_MODIFY_METADATA)
    params = execution.params(params)
    cargs = spec_file_modify_cargs(params, execution)
    ret = spec_file_modify_outputs(params, execution)
    execution.run(cargs)
    return ret


def spec_file_modify(
    spec_file: str,
    add: list[SpecFileModifyAddParamsDict] | None = None,
    remove: list[SpecFileModifyRemoveParamsDict] | None = None,
    runner: Runner | None = None,
) -> SpecFileModifyOutputs:
    """
    ADD OR REMOVE FILES FROM SPEC FILE.
    
    The input spec file is overwritten with the new version. If the spec file
    does not exist, it is created with default metadata.
    
    Removals are done before additions, so that you can remove files with a
    pattern, and then add new files even if they match that removal pattern.
    
    The <structure> argument of -add must match one of the following strings
    (you can use INVALID for cifti or volume files):CORTEX_LEFT
    CORTEX_RIGHT
    CEREBELLUM
    ACCUMBENS_LEFT
    ACCUMBENS_RIGHT
    ALL_GREY_MATTER
    ALL_WHITE_MATTER
    AMYGDALA_LEFT
    AMYGDALA_RIGHT
    BRAIN_STEM
    CAUDATE_LEFT
    CAUDATE_RIGHT
    CEREBELLAR_WHITE_MATTER_LEFT
    CEREBELLAR_WHITE_MATTER_RIGHT
    CEREBELLUM_LEFT
    CEREBELLUM_RIGHT
    CEREBRAL_WHITE_MATTER_LEFT
    CEREBRAL_WHITE_MATTER_RIGHT
    CORTEX
    DIENCEPHALON_VENTRAL_LEFT
    DIENCEPHALON_VENTRAL_RIGHT
    HIPPOCAMPUS_LEFT
    HIPPOCAMPUS_RIGHT
    HIPPOCAMPUS_DENTATE_LEFT
    HIPPOCAMPUS_DENTATE_RIGHT
    INVALID
    OTHER
    OTHER_GREY_MATTER
    OTHER_WHITE_MATTER
    PALLIDUM_LEFT
    PALLIDUM_RIGHT
    PUTAMEN_LEFT
    PUTAMEN_RIGHT
    THALAMUS_LEFT
    THALAMUS_RIGHT
    .
    
    Args:
        spec_file: the spec file to modify.
        add: add a file to the spec.
        remove: remove a file from the spec file.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SpecFileModifyOutputs`).
    """
    params = spec_file_modify_params(
        spec_file=spec_file,
        add=add,
        remove=remove,
    )
    return spec_file_modify_execute(params, runner)


__all__ = [
    "SPEC_FILE_MODIFY_METADATA",
    "SpecFileModifyAddParamsDict",
    "SpecFileModifyAddParamsDictTagged",
    "SpecFileModifyOutputs",
    "SpecFileModifyParamsDict",
    "SpecFileModifyParamsDictTagged",
    "SpecFileModifyRemoveParamsDict",
    "SpecFileModifyRemoveParamsDictTagged",
    "spec_file_modify",
    "spec_file_modify_add",
    "spec_file_modify_execute",
    "spec_file_modify_params",
    "spec_file_modify_remove",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

CIFTI_ROI_AVERAGE_METADATA = Metadata(
    id="1c54604ccdd07eb726448c23d9769ff3ee506272.workbench",
    name="cifti-roi-average",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_CiftiRoiAverageParamsDictNoTag = typing.TypedDict('_CiftiRoiAverageParamsDictNoTag', {
    "cifti-in": InputPathType,
    "text-out": str,
    "roi-vol": typing.NotRequired[InputPathType | None],
    "roi-metric": typing.NotRequired[InputPathType | None],
    "roi-metric": typing.NotRequired[InputPathType | None],
    "roi-metric": typing.NotRequired[InputPathType | None],
    "roi-cifti": typing.NotRequired[InputPathType | None],
})
CiftiRoiAverageParamsDictTagged = typing.TypedDict('CiftiRoiAverageParamsDictTagged', {
    "@type": typing.Literal["workbench/cifti-roi-average"],
    "cifti-in": InputPathType,
    "text-out": str,
    "roi-vol": typing.NotRequired[InputPathType | None],
    "roi-metric": typing.NotRequired[InputPathType | None],
    "roi-metric": typing.NotRequired[InputPathType | None],
    "roi-metric": typing.NotRequired[InputPathType | None],
    "roi-cifti": typing.NotRequired[InputPathType | None],
})
CiftiRoiAverageParamsDict = _CiftiRoiAverageParamsDictNoTag | CiftiRoiAverageParamsDictTagged


class CiftiRoiAverageOutputs(typing.NamedTuple):
    """
    Output object returned when calling `CiftiRoiAverageParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def cifti_roi_average_params(
    cifti_in: InputPathType,
    text_out: str,
    roi_vol: InputPathType | None = None,
    roi_metric: InputPathType | None = None,
    roi_metric_: InputPathType | None = None,
    roi_metric_2: InputPathType | None = None,
    roi_cifti: InputPathType | None = None,
) -> CiftiRoiAverageParamsDictTagged:
    """
    Build parameters.
    
    Args:
        cifti_in: the cifti file to average rows from.
        text_out: output text file of the average values.
        roi_vol: voxels to use\
            \
            the roi volume file.
        roi_metric: vertices to use from cerebellum\
            \
            the cerebellum roi as a metric file.
        roi_metric_: vertices to use from right hemisphere\
            \
            the right roi as a metric file.
        roi_metric_2: vertices to use from left hemisphere\
            \
            the left roi as a metric file.
        roi_cifti: cifti file containing combined rois\
            \
            the rois as a cifti file.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/cifti-roi-average",
        "cifti-in": cifti_in,
        "text-out": text_out,
    }
    if roi_vol is not None:
        params["roi-vol"] = roi_vol
    if roi_metric is not None:
        params["roi-metric"] = roi_metric
    if roi_metric_ is not None:
        params["roi-metric"] = roi_metric_
    if roi_metric_2 is not None:
        params["roi-metric"] = roi_metric_2
    if roi_cifti is not None:
        params["roi-cifti"] = roi_cifti
    return params


def cifti_roi_average_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiRoiAverageParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("cifti-in", None) is None:
        raise StyxValidationError("`cifti-in` must not be None")
    if not isinstance(params["cifti-in"], (pathlib.Path, str)):
        raise StyxValidationError(f'`cifti-in` has the wrong type: Received `{type(params.get("cifti-in", None))}` expected `InputPathType`')
    if params.get("text-out", None) is None:
        raise StyxValidationError("`text-out` must not be None")
    if not isinstance(params["text-out"], str):
        raise StyxValidationError(f'`text-out` has the wrong type: Received `{type(params.get("text-out", None))}` expected `str`')
    if params.get("roi-vol", None) is not None:
        if not isinstance(params["roi-vol"], (pathlib.Path, str)):
            raise StyxValidationError(f'`roi-vol` has the wrong type: Received `{type(params.get("roi-vol", None))}` expected `InputPathType | None`')
    if params.get("roi-metric", None) is not None:
        if not isinstance(params["roi-metric"], (pathlib.Path, str)):
            raise StyxValidationError(f'`roi-metric` has the wrong type: Received `{type(params.get("roi-metric", None))}` expected `InputPathType | None`')
    if params.get("roi-metric", None) is not None:
        if not isinstance(params["roi-metric"], (pathlib.Path, str)):
            raise StyxValidationError(f'`roi-metric` has the wrong type: Received `{type(params.get("roi-metric", None))}` expected `InputPathType | None`')
    if params.get("roi-metric", None) is not None:
        if not isinstance(params["roi-metric"], (pathlib.Path, str)):
            raise StyxValidationError(f'`roi-metric` has the wrong type: Received `{type(params.get("roi-metric", None))}` expected `InputPathType | None`')
    if params.get("roi-cifti", None) is not None:
        if not isinstance(params["roi-cifti"], (pathlib.Path, str)):
            raise StyxValidationError(f'`roi-cifti` has the wrong type: Received `{type(params.get("roi-cifti", None))}` expected `InputPathType | None`')


def cifti_roi_average_cargs(
    params: CiftiRoiAverageParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-cifti-roi-average"
    ])
    cargs.append(execution.input_file(params.get("cifti-in", None)))
    cargs.append(params.get("text-out", None))
    if params.get("roi-vol", None) is not None:
        cargs.extend([
            "-vol-roi",
            execution.input_file(params.get("roi-vol", None))
        ])
    if params.get("roi-metric", None) is not None:
        cargs.extend([
            "-cerebellum-roi",
            execution.input_file(params.get("roi-metric", None))
        ])
    if params.get("roi-metric", None) is not None:
        cargs.extend([
            "-right-roi",
            execution.input_file(params.get("roi-metric", None))
        ])
    if params.get("roi-metric", None) is not None:
        cargs.extend([
            "-left-roi",
            execution.input_file(params.get("roi-metric", None))
        ])
    if params.get("roi-cifti", None) is not None:
        cargs.extend([
            "-cifti-roi",
            execution.input_file(params.get("roi-cifti", None))
        ])
    return cargs


def cifti_roi_average_outputs(
    params: CiftiRoiAverageParamsDict,
    execution: Execution,
) -> CiftiRoiAverageOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = CiftiRoiAverageOutputs(
        root=execution.output_file("."),
    )
    return ret


def cifti_roi_average_execute(
    params: CiftiRoiAverageParamsDict,
    runner: Runner | None = None,
) -> CiftiRoiAverageOutputs:
    """
    AVERAGE ROWS IN A SINGLE CIFTI FILE.
    
    Average the rows that are within the specified ROIs, and write the resulting
    average row to a text file, separated by newlines. If -cifti-roi is
    specified, -left-roi, -right-roi, -cerebellum-roi, and -vol-roi must not be
    specified.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiRoiAverageOutputs`).
    """
    cifti_roi_average_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(CIFTI_ROI_AVERAGE_METADATA)
    params = execution.params(params)
    cargs = cifti_roi_average_cargs(params, execution)
    ret = cifti_roi_average_outputs(params, execution)
    execution.run(cargs)
    return ret


def cifti_roi_average(
    cifti_in: InputPathType,
    text_out: str,
    roi_vol: InputPathType | None = None,
    roi_metric: InputPathType | None = None,
    roi_metric_: InputPathType | None = None,
    roi_metric_2: InputPathType | None = None,
    roi_cifti: InputPathType | None = None,
    runner: Runner | None = None,
) -> CiftiRoiAverageOutputs:
    """
    AVERAGE ROWS IN A SINGLE CIFTI FILE.
    
    Average the rows that are within the specified ROIs, and write the resulting
    average row to a text file, separated by newlines. If -cifti-roi is
    specified, -left-roi, -right-roi, -cerebellum-roi, and -vol-roi must not be
    specified.
    
    Args:
        cifti_in: the cifti file to average rows from.
        text_out: output text file of the average values.
        roi_vol: voxels to use\
            \
            the roi volume file.
        roi_metric: vertices to use from cerebellum\
            \
            the cerebellum roi as a metric file.
        roi_metric_: vertices to use from right hemisphere\
            \
            the right roi as a metric file.
        roi_metric_2: vertices to use from left hemisphere\
            \
            the left roi as a metric file.
        roi_cifti: cifti file containing combined rois\
            \
            the rois as a cifti file.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiRoiAverageOutputs`).
    """
    params = cifti_roi_average_params(
        cifti_in=cifti_in,
        text_out=text_out,
        roi_vol=roi_vol,
        roi_metric=roi_metric,
        roi_metric_=roi_metric_,
        roi_metric_2=roi_metric_2,
        roi_cifti=roi_cifti,
    )
    return cifti_roi_average_execute(params, runner)


__all__ = [
    "CIFTI_ROI_AVERAGE_METADATA",
    "CiftiRoiAverageOutputs",
    "CiftiRoiAverageParamsDict",
    "CiftiRoiAverageParamsDictTagged",
    "cifti_roi_average",
    "cifti_roi_average_execute",
    "cifti_roi_average_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

METRIC_FALSE_CORRELATION_METADATA = Metadata(
    id="c5fa55ac073dd0a51eb97deb240c610fa9ebef79.workbench",
    name="metric-false-correlation",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_MetricFalseCorrelationParamsDictNoTag = typing.TypedDict('_MetricFalseCorrelationParamsDictNoTag', {
    "surface": InputPathType,
    "metric-in": InputPathType,
    "3D-dist": float,
    "geo-outer": float,
    "geo-inner": float,
    "metric-out": str,
    "text-out": typing.NotRequired[str | None],
    "roi-metric": typing.NotRequired[InputPathType | None],
})
MetricFalseCorrelationParamsDictTagged = typing.TypedDict('MetricFalseCorrelationParamsDictTagged', {
    "@type": typing.Literal["workbench/metric-false-correlation"],
    "surface": InputPathType,
    "metric-in": InputPathType,
    "3D-dist": float,
    "geo-outer": float,
    "geo-inner": float,
    "metric-out": str,
    "text-out": typing.NotRequired[str | None],
    "roi-metric": typing.NotRequired[InputPathType | None],
})
MetricFalseCorrelationParamsDict = _MetricFalseCorrelationParamsDictNoTag | MetricFalseCorrelationParamsDictTagged


class MetricFalseCorrelationOutputs(typing.NamedTuple):
    """
    Output object returned when calling `MetricFalseCorrelationParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    metric_out: OutputPathType
    """the output metric"""


def metric_false_correlation_params(
    surface: InputPathType,
    metric_in: InputPathType,
    v_3_d_dist: float,
    geo_outer: float,
    geo_inner: float,
    metric_out: str,
    text_out: str | None = None,
    roi_metric: InputPathType | None = None,
) -> MetricFalseCorrelationParamsDictTagged:
    """
    Build parameters.
    
    Args:
        surface: the surface to compute geodesic and 3D distance with.
        metric_in: the metric to correlate.
        v_3_d_dist: maximum 3D distance to check around each vertex.
        geo_outer: maximum geodesic distance to use for neighboring correlation.
        geo_inner: minimum geodesic distance to use for neighboring correlation.
        metric_out: the output metric.
        text_out: dump the raw measures used to a text file\
            \
            the output text file.
        roi_metric: select a region of interest that has data\
            \
            the region, as a metric file.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/metric-false-correlation",
        "surface": surface,
        "metric-in": metric_in,
        "3D-dist": v_3_d_dist,
        "geo-outer": geo_outer,
        "geo-inner": geo_inner,
        "metric-out": metric_out,
    }
    if text_out is not None:
        params["text-out"] = text_out
    if roi_metric is not None:
        params["roi-metric"] = roi_metric
    return params


def metric_false_correlation_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `MetricFalseCorrelationParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("surface", None) is None:
        raise StyxValidationError("`surface` must not be None")
    if not isinstance(params["surface"], (pathlib.Path, str)):
        raise StyxValidationError(f'`surface` has the wrong type: Received `{type(params.get("surface", None))}` expected `InputPathType`')
    if params.get("metric-in", None) is None:
        raise StyxValidationError("`metric-in` must not be None")
    if not isinstance(params["metric-in"], (pathlib.Path, str)):
        raise StyxValidationError(f'`metric-in` has the wrong type: Received `{type(params.get("metric-in", None))}` expected `InputPathType`')
    if params.get("3D-dist", None) is None:
        raise StyxValidationError("`3D-dist` must not be None")
    if not isinstance(params["3D-dist"], (float, int)):
        raise StyxValidationError(f'`3D-dist` has the wrong type: Received `{type(params.get("3D-dist", None))}` expected `float`')
    if params.get("geo-outer", None) is None:
        raise StyxValidationError("`geo-outer` must not be None")
    if not isinstance(params["geo-outer"], (float, int)):
        raise StyxValidationError(f'`geo-outer` has the wrong type: Received `{type(params.get("geo-outer", None))}` expected `float`')
    if params.get("geo-inner", None) is None:
        raise StyxValidationError("`geo-inner` must not be None")
    if not isinstance(params["geo-inner"], (float, int)):
        raise StyxValidationError(f'`geo-inner` has the wrong type: Received `{type(params.get("geo-inner", None))}` expected `float`')
    if params.get("metric-out", None) is None:
        raise StyxValidationError("`metric-out` must not be None")
    if not isinstance(params["metric-out"], str):
        raise StyxValidationError(f'`metric-out` has the wrong type: Received `{type(params.get("metric-out", None))}` expected `str`')
    if params.get("text-out", None) is not None:
        if not isinstance(params["text-out"], str):
            raise StyxValidationError(f'`text-out` has the wrong type: Received `{type(params.get("text-out", None))}` expected `str | None`')
    if params.get("roi-metric", None) is not None:
        if not isinstance(params["roi-metric"], (pathlib.Path, str)):
            raise StyxValidationError(f'`roi-metric` has the wrong type: Received `{type(params.get("roi-metric", None))}` expected `InputPathType | None`')


def metric_false_correlation_cargs(
    params: MetricFalseCorrelationParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-metric-false-correlation"
    ])
    cargs.append(execution.input_file(params.get("surface", None)))
    cargs.append(execution.input_file(params.get("metric-in", None)))
    cargs.append(str(params.get("3D-dist", None)))
    cargs.append(str(params.get("geo-outer", None)))
    cargs.append(str(params.get("geo-inner", None)))
    cargs.append(params.get("metric-out", None))
    if params.get("text-out", None) is not None:
        cargs.extend([
            "-dump-text",
            params.get("text-out", None)
        ])
    if params.get("roi-metric", None) is not None:
        cargs.extend([
            "-roi",
            execution.input_file(params.get("roi-metric", None))
        ])
    return cargs


def metric_false_correlation_outputs(
    params: MetricFalseCorrelationParamsDict,
    execution: Execution,
) -> MetricFalseCorrelationOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = MetricFalseCorrelationOutputs(
        root=execution.output_file("."),
        metric_out=execution.output_file(params.get("metric-out", None)),
    )
    return ret


def metric_false_correlation_execute(
    params: MetricFalseCorrelationParamsDict,
    runner: Runner | None = None,
) -> MetricFalseCorrelationOutputs:
    """
    COMPARE CORRELATION LOCALLY AND ACROSS/THROUGH SULCI/GYRI.
    
    For each vertex, compute the average correlation within a range of geodesic
    distances that don't cross a sulcus/gyrus, and the correlation to the
    closest vertex crossing a sulcus/gyrus. A vertex is considered to cross a
    sulcus/gyrus if the 3D distance is less than a third of the geodesic
    distance. The output file contains the ratio between these correlations, and
    some additional maps to help explain the ratio.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MetricFalseCorrelationOutputs`).
    """
    metric_false_correlation_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(METRIC_FALSE_CORRELATION_METADATA)
    params = execution.params(params)
    cargs = metric_false_correlation_cargs(params, execution)
    ret = metric_false_correlation_outputs(params, execution)
    execution.run(cargs)
    return ret


def metric_false_correlation(
    surface: InputPathType,
    metric_in: InputPathType,
    v_3_d_dist: float,
    geo_outer: float,
    geo_inner: float,
    metric_out: str,
    text_out: str | None = None,
    roi_metric: InputPathType | None = None,
    runner: Runner | None = None,
) -> MetricFalseCorrelationOutputs:
    """
    COMPARE CORRELATION LOCALLY AND ACROSS/THROUGH SULCI/GYRI.
    
    For each vertex, compute the average correlation within a range of geodesic
    distances that don't cross a sulcus/gyrus, and the correlation to the
    closest vertex crossing a sulcus/gyrus. A vertex is considered to cross a
    sulcus/gyrus if the 3D distance is less than a third of the geodesic
    distance. The output file contains the ratio between these correlations, and
    some additional maps to help explain the ratio.
    
    Args:
        surface: the surface to compute geodesic and 3D distance with.
        metric_in: the metric to correlate.
        v_3_d_dist: maximum 3D distance to check around each vertex.
        geo_outer: maximum geodesic distance to use for neighboring correlation.
        geo_inner: minimum geodesic distance to use for neighboring correlation.
        metric_out: the output metric.
        text_out: dump the raw measures used to a text file\
            \
            the output text file.
        roi_metric: select a region of interest that has data\
            \
            the region, as a metric file.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MetricFalseCorrelationOutputs`).
    """
    params = metric_false_correlation_params(
        surface=surface,
        metric_in=metric_in,
        v_3_d_dist=v_3_d_dist,
        geo_outer=geo_outer,
        geo_inner=geo_inner,
        metric_out=metric_out,
        text_out=text_out,
        roi_metric=roi_metric,
    )
    return metric_false_correlation_execute(params, runner)


__all__ = [
    "METRIC_FALSE_CORRELATION_METADATA",
    "MetricFalseCorrelationOutputs",
    "MetricFalseCorrelationParamsDict",
    "MetricFalseCorrelationParamsDictTagged",
    "metric_false_correlation",
    "metric_false_correlation_execute",
    "metric_false_correlation_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

CIFTI_CREATE_DENSE_SCALAR_METADATA = Metadata(
    id="e17e9d7682a88404f1c7e8f47d1454ee199d7b6e.workbench",
    name="cifti-create-dense-scalar",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_CiftiCreateDenseScalarVolumeParamsDictNoTag = typing.TypedDict('_CiftiCreateDenseScalarVolumeParamsDictNoTag', {
    "volume-data": InputPathType,
    "structure-label-volume": InputPathType,
})
CiftiCreateDenseScalarVolumeParamsDictTagged = typing.TypedDict('CiftiCreateDenseScalarVolumeParamsDictTagged', {
    "@type": typing.Literal["volume"],
    "volume-data": InputPathType,
    "structure-label-volume": InputPathType,
})
CiftiCreateDenseScalarVolumeParamsDict = _CiftiCreateDenseScalarVolumeParamsDictNoTag | CiftiCreateDenseScalarVolumeParamsDictTagged


_CiftiCreateDenseScalarLeftMetricParamsDictNoTag = typing.TypedDict('_CiftiCreateDenseScalarLeftMetricParamsDictNoTag', {
    "metric": InputPathType,
    "roi-metric": typing.NotRequired[InputPathType | None],
})
CiftiCreateDenseScalarLeftMetricParamsDictTagged = typing.TypedDict('CiftiCreateDenseScalarLeftMetricParamsDictTagged', {
    "@type": typing.Literal["left-metric"],
    "metric": InputPathType,
    "roi-metric": typing.NotRequired[InputPathType | None],
})
CiftiCreateDenseScalarLeftMetricParamsDict = _CiftiCreateDenseScalarLeftMetricParamsDictNoTag | CiftiCreateDenseScalarLeftMetricParamsDictTagged


_CiftiCreateDenseScalarRightMetricParamsDictNoTag = typing.TypedDict('_CiftiCreateDenseScalarRightMetricParamsDictNoTag', {
    "metric": InputPathType,
    "roi-metric": typing.NotRequired[InputPathType | None],
})
CiftiCreateDenseScalarRightMetricParamsDictTagged = typing.TypedDict('CiftiCreateDenseScalarRightMetricParamsDictTagged', {
    "@type": typing.Literal["right-metric"],
    "metric": InputPathType,
    "roi-metric": typing.NotRequired[InputPathType | None],
})
CiftiCreateDenseScalarRightMetricParamsDict = _CiftiCreateDenseScalarRightMetricParamsDictNoTag | CiftiCreateDenseScalarRightMetricParamsDictTagged


_CiftiCreateDenseScalarCerebellumMetricParamsDictNoTag = typing.TypedDict('_CiftiCreateDenseScalarCerebellumMetricParamsDictNoTag', {
    "metric": InputPathType,
    "roi-metric": typing.NotRequired[InputPathType | None],
})
CiftiCreateDenseScalarCerebellumMetricParamsDictTagged = typing.TypedDict('CiftiCreateDenseScalarCerebellumMetricParamsDictTagged', {
    "@type": typing.Literal["cerebellum-metric"],
    "metric": InputPathType,
    "roi-metric": typing.NotRequired[InputPathType | None],
})
CiftiCreateDenseScalarCerebellumMetricParamsDict = _CiftiCreateDenseScalarCerebellumMetricParamsDictNoTag | CiftiCreateDenseScalarCerebellumMetricParamsDictTagged


_CiftiCreateDenseScalarMetricParamsDictNoTag = typing.TypedDict('_CiftiCreateDenseScalarMetricParamsDictNoTag', {
    "structure": str,
    "metric": InputPathType,
    "roi-metric": typing.NotRequired[InputPathType | None],
})
CiftiCreateDenseScalarMetricParamsDictTagged = typing.TypedDict('CiftiCreateDenseScalarMetricParamsDictTagged', {
    "@type": typing.Literal["metric"],
    "structure": str,
    "metric": InputPathType,
    "roi-metric": typing.NotRequired[InputPathType | None],
})
CiftiCreateDenseScalarMetricParamsDict = _CiftiCreateDenseScalarMetricParamsDictNoTag | CiftiCreateDenseScalarMetricParamsDictTagged


_CiftiCreateDenseScalarParamsDictNoTag = typing.TypedDict('_CiftiCreateDenseScalarParamsDictNoTag', {
    "cifti-out": str,
    "volume": typing.NotRequired[CiftiCreateDenseScalarVolumeParamsDict | None],
    "left-metric": typing.NotRequired[CiftiCreateDenseScalarLeftMetricParamsDict | None],
    "right-metric": typing.NotRequired[CiftiCreateDenseScalarRightMetricParamsDict | None],
    "cerebellum-metric": typing.NotRequired[CiftiCreateDenseScalarCerebellumMetricParamsDict | None],
    "metric": typing.NotRequired[list[CiftiCreateDenseScalarMetricParamsDict] | None],
    "file": typing.NotRequired[str | None],
})
CiftiCreateDenseScalarParamsDictTagged = typing.TypedDict('CiftiCreateDenseScalarParamsDictTagged', {
    "@type": typing.Literal["workbench/cifti-create-dense-scalar"],
    "cifti-out": str,
    "volume": typing.NotRequired[CiftiCreateDenseScalarVolumeParamsDict | None],
    "left-metric": typing.NotRequired[CiftiCreateDenseScalarLeftMetricParamsDict | None],
    "right-metric": typing.NotRequired[CiftiCreateDenseScalarRightMetricParamsDict | None],
    "cerebellum-metric": typing.NotRequired[CiftiCreateDenseScalarCerebellumMetricParamsDict | None],
    "metric": typing.NotRequired[list[CiftiCreateDenseScalarMetricParamsDict] | None],
    "file": typing.NotRequired[str | None],
})
CiftiCreateDenseScalarParamsDict = _CiftiCreateDenseScalarParamsDictNoTag | CiftiCreateDenseScalarParamsDictTagged


def cifti_create_dense_scalar_volume(
    volume_data: InputPathType,
    structure_label_volume: InputPathType,
) -> CiftiCreateDenseScalarVolumeParamsDictTagged:
    """
    Build parameters.
    
    Args:
        volume_data: volume file containing all voxel data for all volume\
            structures.
        structure_label_volume: label volume file containing labels for cifti\
            structures.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "volume",
        "volume-data": volume_data,
        "structure-label-volume": structure_label_volume,
    }
    return params


def cifti_create_dense_scalar_volume_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiCreateDenseScalarVolumeParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("volume-data", None) is None:
        raise StyxValidationError("`volume-data` must not be None")
    if not isinstance(params["volume-data"], (pathlib.Path, str)):
        raise StyxValidationError(f'`volume-data` has the wrong type: Received `{type(params.get("volume-data", None))}` expected `InputPathType`')
    if params.get("structure-label-volume", None) is None:
        raise StyxValidationError("`structure-label-volume` must not be None")
    if not isinstance(params["structure-label-volume"], (pathlib.Path, str)):
        raise StyxValidationError(f'`structure-label-volume` has the wrong type: Received `{type(params.get("structure-label-volume", None))}` expected `InputPathType`')


def cifti_create_dense_scalar_volume_cargs(
    params: CiftiCreateDenseScalarVolumeParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-volume",
        execution.input_file(params.get("volume-data", None)),
        execution.input_file(params.get("structure-label-volume", None))
    ])
    return cargs


def cifti_create_dense_scalar_left_metric(
    metric: InputPathType,
    roi_metric: InputPathType | None = None,
) -> CiftiCreateDenseScalarLeftMetricParamsDictTagged:
    """
    Build parameters.
    
    Args:
        metric: the metric file.
        roi_metric: roi of vertices to use from left surface\
            \
            the ROI as a metric file.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "left-metric",
        "metric": metric,
    }
    if roi_metric is not None:
        params["roi-metric"] = roi_metric
    return params


def cifti_create_dense_scalar_left_metric_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiCreateDenseScalarLeftMetricParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("metric", None) is None:
        raise StyxValidationError("`metric` must not be None")
    if not isinstance(params["metric"], (pathlib.Path, str)):
        raise StyxValidationError(f'`metric` has the wrong type: Received `{type(params.get("metric", None))}` expected `InputPathType`')
    if params.get("roi-metric", None) is not None:
        if not isinstance(params["roi-metric"], (pathlib.Path, str)):
            raise StyxValidationError(f'`roi-metric` has the wrong type: Received `{type(params.get("roi-metric", None))}` expected `InputPathType | None`')


def cifti_create_dense_scalar_left_metric_cargs(
    params: CiftiCreateDenseScalarLeftMetricParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-left-metric",
        execution.input_file(params.get("metric", None))
    ])
    if params.get("roi-metric", None) is not None:
        cargs.extend([
            "-roi-left",
            execution.input_file(params.get("roi-metric", None))
        ])
    return cargs


def cifti_create_dense_scalar_right_metric(
    metric: InputPathType,
    roi_metric: InputPathType | None = None,
) -> CiftiCreateDenseScalarRightMetricParamsDictTagged:
    """
    Build parameters.
    
    Args:
        metric: the metric file.
        roi_metric: roi of vertices to use from right surface\
            \
            the ROI as a metric file.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "right-metric",
        "metric": metric,
    }
    if roi_metric is not None:
        params["roi-metric"] = roi_metric
    return params


def cifti_create_dense_scalar_right_metric_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiCreateDenseScalarRightMetricParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("metric", None) is None:
        raise StyxValidationError("`metric` must not be None")
    if not isinstance(params["metric"], (pathlib.Path, str)):
        raise StyxValidationError(f'`metric` has the wrong type: Received `{type(params.get("metric", None))}` expected `InputPathType`')
    if params.get("roi-metric", None) is not None:
        if not isinstance(params["roi-metric"], (pathlib.Path, str)):
            raise StyxValidationError(f'`roi-metric` has the wrong type: Received `{type(params.get("roi-metric", None))}` expected `InputPathType | None`')


def cifti_create_dense_scalar_right_metric_cargs(
    params: CiftiCreateDenseScalarRightMetricParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-right-metric",
        execution.input_file(params.get("metric", None))
    ])
    if params.get("roi-metric", None) is not None:
        cargs.extend([
            "-roi-right",
            execution.input_file(params.get("roi-metric", None))
        ])
    return cargs


def cifti_create_dense_scalar_cerebellum_metric(
    metric: InputPathType,
    roi_metric: InputPathType | None = None,
) -> CiftiCreateDenseScalarCerebellumMetricParamsDictTagged:
    """
    Build parameters.
    
    Args:
        metric: the metric file.
        roi_metric: roi of vertices to use from right surface\
            \
            the ROI as a metric file.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "cerebellum-metric",
        "metric": metric,
    }
    if roi_metric is not None:
        params["roi-metric"] = roi_metric
    return params


def cifti_create_dense_scalar_cerebellum_metric_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiCreateDenseScalarCerebellumMetricParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("metric", None) is None:
        raise StyxValidationError("`metric` must not be None")
    if not isinstance(params["metric"], (pathlib.Path, str)):
        raise StyxValidationError(f'`metric` has the wrong type: Received `{type(params.get("metric", None))}` expected `InputPathType`')
    if params.get("roi-metric", None) is not None:
        if not isinstance(params["roi-metric"], (pathlib.Path, str)):
            raise StyxValidationError(f'`roi-metric` has the wrong type: Received `{type(params.get("roi-metric", None))}` expected `InputPathType | None`')


def cifti_create_dense_scalar_cerebellum_metric_cargs(
    params: CiftiCreateDenseScalarCerebellumMetricParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-cerebellum-metric",
        execution.input_file(params.get("metric", None))
    ])
    if params.get("roi-metric", None) is not None:
        cargs.extend([
            "-roi-cerebellum",
            execution.input_file(params.get("roi-metric", None))
        ])
    return cargs


def cifti_create_dense_scalar_metric(
    structure: str,
    metric: InputPathType,
    roi_metric: InputPathType | None = None,
) -> CiftiCreateDenseScalarMetricParamsDictTagged:
    """
    Build parameters.
    
    Args:
        structure: the structure name.
        metric: the metric file.
        roi_metric: roi of vertices to use from this structure\
            \
            the ROI as a metric file.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "metric",
        "structure": structure,
        "metric": metric,
    }
    if roi_metric is not None:
        params["roi-metric"] = roi_metric
    return params


def cifti_create_dense_scalar_metric_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiCreateDenseScalarMetricParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("structure", None) is None:
        raise StyxValidationError("`structure` must not be None")
    if not isinstance(params["structure"], str):
        raise StyxValidationError(f'`structure` has the wrong type: Received `{type(params.get("structure", None))}` expected `str`')
    if params.get("metric", None) is None:
        raise StyxValidationError("`metric` must not be None")
    if not isinstance(params["metric"], (pathlib.Path, str)):
        raise StyxValidationError(f'`metric` has the wrong type: Received `{type(params.get("metric", None))}` expected `InputPathType`')
    if params.get("roi-metric", None) is not None:
        if not isinstance(params["roi-metric"], (pathlib.Path, str)):
            raise StyxValidationError(f'`roi-metric` has the wrong type: Received `{type(params.get("roi-metric", None))}` expected `InputPathType | None`')


def cifti_create_dense_scalar_metric_cargs(
    params: CiftiCreateDenseScalarMetricParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-metric",
        params.get("structure", None),
        execution.input_file(params.get("metric", None))
    ])
    if params.get("roi-metric", None) is not None:
        cargs.extend([
            "-roi",
            execution.input_file(params.get("roi-metric", None))
        ])
    return cargs


class CiftiCreateDenseScalarOutputs(typing.NamedTuple):
    """
    Output object returned when calling `CiftiCreateDenseScalarParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    cifti_out: OutputPathType
    """the output cifti file"""


def cifti_create_dense_scalar_params(
    cifti_out: str,
    volume: CiftiCreateDenseScalarVolumeParamsDict | None = None,
    left_metric: CiftiCreateDenseScalarLeftMetricParamsDict | None = None,
    right_metric: CiftiCreateDenseScalarRightMetricParamsDict | None = None,
    cerebellum_metric: CiftiCreateDenseScalarCerebellumMetricParamsDict | None = None,
    metric: list[CiftiCreateDenseScalarMetricParamsDict] | None = None,
    file: str | None = None,
) -> CiftiCreateDenseScalarParamsDictTagged:
    """
    Build parameters.
    
    Args:
        cifti_out: the output cifti file.
        volume: volume component.
        left_metric: metric for the left cortical surface.
        right_metric: metric for the right cortical surface.
        cerebellum_metric: metric for the cerebellum.
        metric: metric for a specified surface structure.
        file: use a text file to set all map names\
            \
            text file containing map names, one per line.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/cifti-create-dense-scalar",
        "cifti-out": cifti_out,
    }
    if volume is not None:
        params["volume"] = volume
    if left_metric is not None:
        params["left-metric"] = left_metric
    if right_metric is not None:
        params["right-metric"] = right_metric
    if cerebellum_metric is not None:
        params["cerebellum-metric"] = cerebellum_metric
    if metric is not None:
        params["metric"] = metric
    if file is not None:
        params["file"] = file
    return params


def cifti_create_dense_scalar_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiCreateDenseScalarParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("cifti-out", None) is None:
        raise StyxValidationError("`cifti-out` must not be None")
    if not isinstance(params["cifti-out"], str):
        raise StyxValidationError(f'`cifti-out` has the wrong type: Received `{type(params.get("cifti-out", None))}` expected `str`')
    if params.get("volume", None) is not None:
        cifti_create_dense_scalar_volume_validate(params["volume"])
    if params.get("left-metric", None) is not None:
        cifti_create_dense_scalar_left_metric_validate(params["left-metric"])
    if params.get("right-metric", None) is not None:
        cifti_create_dense_scalar_right_metric_validate(params["right-metric"])
    if params.get("cerebellum-metric", None) is not None:
        cifti_create_dense_scalar_cerebellum_metric_validate(params["cerebellum-metric"])
    if params.get("metric", None) is not None:
        if not isinstance(params["metric"], list):
            raise StyxValidationError(f'`metric` has the wrong type: Received `{type(params.get("metric", None))}` expected `list[CiftiCreateDenseScalarMetricParamsDict] | None`')
        for e in params["metric"]:
            cifti_create_dense_scalar_metric_validate(e)
    if params.get("file", None) is not None:
        if not isinstance(params["file"], str):
            raise StyxValidationError(f'`file` has the wrong type: Received `{type(params.get("file", None))}` expected `str | None`')


def cifti_create_dense_scalar_cargs(
    params: CiftiCreateDenseScalarParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-cifti-create-dense-scalar"
    ])
    cargs.append(params.get("cifti-out", None))
    if params.get("volume", None) is not None or params.get("left-metric", None) is not None or params.get("right-metric", None) is not None or params.get("cerebellum-metric", None) is not None or params.get("metric", None) is not None:
        cargs.extend([
            *(cifti_create_dense_scalar_volume_cargs(params.get("volume", None), execution) if (params.get("volume", None) is not None) else []),
            *(cifti_create_dense_scalar_left_metric_cargs(params.get("left-metric", None), execution) if (params.get("left-metric", None) is not None) else []),
            *(cifti_create_dense_scalar_right_metric_cargs(params.get("right-metric", None), execution) if (params.get("right-metric", None) is not None) else []),
            *(cifti_create_dense_scalar_cerebellum_metric_cargs(params.get("cerebellum-metric", None), execution) if (params.get("cerebellum-metric", None) is not None) else []),
            *([a for c in [cifti_create_dense_scalar_metric_cargs(s, execution) for s in params.get("metric", None)] for a in c] if (params.get("metric", None) is not None) else [])
        ])
    if params.get("file", None) is not None:
        cargs.extend([
            "-name-file",
            params.get("file", None)
        ])
    return cargs


def cifti_create_dense_scalar_outputs(
    params: CiftiCreateDenseScalarParamsDict,
    execution: Execution,
) -> CiftiCreateDenseScalarOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = CiftiCreateDenseScalarOutputs(
        root=execution.output_file("."),
        cifti_out=execution.output_file(params.get("cifti-out", None)),
    )
    return ret


def cifti_create_dense_scalar_execute(
    params: CiftiCreateDenseScalarParamsDict,
    runner: Runner | None = None,
) -> CiftiCreateDenseScalarOutputs:
    """
    CREATE A CIFTI DENSE SCALAR FILE.
    
    All input files must have the same number of columns/subvolumes. Only the
    specified components will be in the output cifti file. Map names will be
    taken from one of the input files. At least one component must be specified.
    
    See -volume-label-import and -volume-help for format details of label volume
    files. The -metric structure argument and labels in the
    structure-label-volume must use names from the below list (labels with other
    names are ignored, -metric arguments with other strings are an error):
    
    CORTEX_LEFT
    CORTEX_RIGHT
    CEREBELLUM
    ACCUMBENS_LEFT
    ACCUMBENS_RIGHT
    ALL_GREY_MATTER
    ALL_WHITE_MATTER
    AMYGDALA_LEFT
    AMYGDALA_RIGHT
    BRAIN_STEM
    CAUDATE_LEFT
    CAUDATE_RIGHT
    CEREBELLAR_WHITE_MATTER_LEFT
    CEREBELLAR_WHITE_MATTER_RIGHT
    CEREBELLUM_LEFT
    CEREBELLUM_RIGHT
    CEREBRAL_WHITE_MATTER_LEFT
    CEREBRAL_WHITE_MATTER_RIGHT
    CORTEX
    DIENCEPHALON_VENTRAL_LEFT
    DIENCEPHALON_VENTRAL_RIGHT
    HIPPOCAMPUS_LEFT
    HIPPOCAMPUS_RIGHT
    HIPPOCAMPUS_DENTATE_LEFT
    HIPPOCAMPUS_DENTATE_RIGHT
    INVALID
    OTHER
    OTHER_GREY_MATTER
    OTHER_WHITE_MATTER
    PALLIDUM_LEFT
    PALLIDUM_RIGHT
    PUTAMEN_LEFT
    PUTAMEN_RIGHT
    THALAMUS_LEFT
    THALAMUS_RIGHT.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiCreateDenseScalarOutputs`).
    """
    cifti_create_dense_scalar_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(CIFTI_CREATE_DENSE_SCALAR_METADATA)
    params = execution.params(params)
    cargs = cifti_create_dense_scalar_cargs(params, execution)
    ret = cifti_create_dense_scalar_outputs(params, execution)
    execution.run(cargs)
    return ret


def cifti_create_dense_scalar(
    cifti_out: str,
    volume: CiftiCreateDenseScalarVolumeParamsDict | None = None,
    left_metric: CiftiCreateDenseScalarLeftMetricParamsDict | None = None,
    right_metric: CiftiCreateDenseScalarRightMetricParamsDict | None = None,
    cerebellum_metric: CiftiCreateDenseScalarCerebellumMetricParamsDict | None = None,
    metric: list[CiftiCreateDenseScalarMetricParamsDict] | None = None,
    file: str | None = None,
    runner: Runner | None = None,
) -> CiftiCreateDenseScalarOutputs:
    """
    CREATE A CIFTI DENSE SCALAR FILE.
    
    All input files must have the same number of columns/subvolumes. Only the
    specified components will be in the output cifti file. Map names will be
    taken from one of the input files. At least one component must be specified.
    
    See -volume-label-import and -volume-help for format details of label volume
    files. The -metric structure argument and labels in the
    structure-label-volume must use names from the below list (labels with other
    names are ignored, -metric arguments with other strings are an error):
    
    CORTEX_LEFT
    CORTEX_RIGHT
    CEREBELLUM
    ACCUMBENS_LEFT
    ACCUMBENS_RIGHT
    ALL_GREY_MATTER
    ALL_WHITE_MATTER
    AMYGDALA_LEFT
    AMYGDALA_RIGHT
    BRAIN_STEM
    CAUDATE_LEFT
    CAUDATE_RIGHT
    CEREBELLAR_WHITE_MATTER_LEFT
    CEREBELLAR_WHITE_MATTER_RIGHT
    CEREBELLUM_LEFT
    CEREBELLUM_RIGHT
    CEREBRAL_WHITE_MATTER_LEFT
    CEREBRAL_WHITE_MATTER_RIGHT
    CORTEX
    DIENCEPHALON_VENTRAL_LEFT
    DIENCEPHALON_VENTRAL_RIGHT
    HIPPOCAMPUS_LEFT
    HIPPOCAMPUS_RIGHT
    HIPPOCAMPUS_DENTATE_LEFT
    HIPPOCAMPUS_DENTATE_RIGHT
    INVALID
    OTHER
    OTHER_GREY_MATTER
    OTHER_WHITE_MATTER
    PALLIDUM_LEFT
    PALLIDUM_RIGHT
    PUTAMEN_LEFT
    PUTAMEN_RIGHT
    THALAMUS_LEFT
    THALAMUS_RIGHT.
    
    Args:
        cifti_out: the output cifti file.
        volume: volume component.
        left_metric: metric for the left cortical surface.
        right_metric: metric for the right cortical surface.
        cerebellum_metric: metric for the cerebellum.
        metric: metric for a specified surface structure.
        file: use a text file to set all map names\
            \
            text file containing map names, one per line.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiCreateDenseScalarOutputs`).
    """
    params = cifti_create_dense_scalar_params(
        cifti_out=cifti_out,
        volume=volume,
        left_metric=left_metric,
        right_metric=right_metric,
        cerebellum_metric=cerebellum_metric,
        metric=metric,
        file=file,
    )
    return cifti_create_dense_scalar_execute(params, runner)


__all__ = [
    "CIFTI_CREATE_DENSE_SCALAR_METADATA",
    "CiftiCreateDenseScalarCerebellumMetricParamsDict",
    "CiftiCreateDenseScalarCerebellumMetricParamsDictTagged",
    "CiftiCreateDenseScalarLeftMetricParamsDict",
    "CiftiCreateDenseScalarLeftMetricParamsDictTagged",
    "CiftiCreateDenseScalarMetricParamsDict",
    "CiftiCreateDenseScalarMetricParamsDictTagged",
    "CiftiCreateDenseScalarOutputs",
    "CiftiCreateDenseScalarParamsDict",
    "CiftiCreateDenseScalarParamsDictTagged",
    "CiftiCreateDenseScalarRightMetricParamsDict",
    "CiftiCreateDenseScalarRightMetricParamsDictTagged",
    "CiftiCreateDenseScalarVolumeParamsDict",
    "CiftiCreateDenseScalarVolumeParamsDictTagged",
    "cifti_create_dense_scalar",
    "cifti_create_dense_scalar_cerebellum_metric",
    "cifti_create_dense_scalar_execute",
    "cifti_create_dense_scalar_left_metric",
    "cifti_create_dense_scalar_metric",
    "cifti_create_dense_scalar_params",
    "cifti_create_dense_scalar_right_metric",
    "cifti_create_dense_scalar_volume",
]
